@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -595,7 +595,7 @@ void CMainWindow::readSettings()
     // first try new settings
     {
       QSettings settings("Graphviz", "gvedit");
-      if (settings.contains("pos") and settings.contains("size")) {
+      if (settings.contains("pos") && settings.contains("size")) {
         QPoint pos = settings.value("pos", QPoint(200, 200)).toPoint();
         QSize size = settings.value("size", QSize(400, 400)).toSize();
         move(pos);
