@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -14,6 +14,7 @@
 #include <math.h>
 #include <sparse/QuadTree.h>
 #include <stdbool.h>
+#include <stddef.h>
 #include <string.h>
 #include <cgraph/agxbuf.h>
 #include <cgraph/alloc.h>
@@ -381,22 +382,29 @@ void plot_dot_map(Agraph_t* gr, int n, int dim, double *x, SparseMatrix polys,
   agxbfree(&sbuff);
 }
 
-static void get_tri(int n, int dim, double *x, int *nt, struct Triangle **T, SparseMatrix *E) {
-   /* always contains a self edge and is symmetric.
-     input:
-     n: number of points
-     dim: dimension, only first2D used
-     x: point j is stored x[j*dim]--x[j*dim+dim-1]
-     output: 
-     nt: number of triangles
-     T: triangles
-     E: a matrix of size nxn, if two points i > j are connected by an triangulation edge, and is neighboring two triangles
-     .  t1 and t2, then A[i,j] is both t1 and t2
-   */
+/** construct triangles
+ *
+ * Always contains a self edge and is symmetric.
+ *
+ * @param n Number of points
+ * @param dim Dimension, only first2D is used
+ * @param x Point j is stored x[j × dim] -- x[j × dim + dim - 1]
+ * @param nt [out] Number of triangles
+ * @param T [out] triangles
+ * @param E [out] A matrix of size n×n, if two points i > j are connected by an
+ *   triangulation edge, and is neighboring two triangles t1 and t2, then
+ *   A[i, j] is both t1 and t2
+ * @return 0 on success
+ */
+static int get_tri(int n, int dim, double *x, int *nt, struct Triangle **T,
+                   SparseMatrix *E) {
   int i, j, i0, i1, i2, ntri;
   SparseMatrix A, B;
 
   int* trilist = get_triangles(x, n, &ntri);
+  if (trilist == NULL) {
+    return -1;
+  }
 
   *T = gv_calloc(ntri, sizeof(struct Triangle));
 
@@ -421,6 +429,7 @@ static void get_tri(int n, int dim, double *x, int *nt, struct Triangle **T, Spa
   *nt = ntri;
 
   free(trilist);
+  return 0;
 }
 
 static SparseMatrix get_country_graph(int n, SparseMatrix A, int *groups, int GRP_RANDOM, int GRP_BBOX){
@@ -1032,8 +1041,8 @@ static void make_map_internal(bool include_OK_points,
 
   for (i = 0; i < n; i++){
     for (j = 0; j < dim2; j++) {
-      xmax[j] = MAX(xmax[j], x[i*dim+j]);
-      xmin[j] = MIN(xmin[j], x[i*dim+j]);
+      xmax[j] = fmax(xmax[j], x[i*dim+j]);
+      xmin[j] = fmin(xmin[j], x[i*dim+j]);
     }
   }
   boxsize[0] = xmax[0] - xmin[0];
@@ -1108,8 +1117,8 @@ static void make_map_internal(bool include_OK_points,
 	xmin[i] -= boxsize[i]*(-bounding_box_margin);
 	xmax[i] += boxsize[i]*(-bounding_box_margin);
       } else { // auto bounding box
-	xmin[i] -= MAX(boxsize[i]*0.2, 2.*shore_depth_tol);
-	xmax[i] += MAX(boxsize[i]*0.2, 2*shore_depth_tol);
+	xmin[i] -= fmax(boxsize[i] * 0.2, 2.* shore_depth_tol);
+	xmax[i] += fmax(boxsize[i] * 0.2, 2 * shore_depth_tol);
       }
     }
     if (Verbose) {
@@ -1119,14 +1128,15 @@ static void make_map_internal(bool include_OK_points,
       else if (bbm < 0)
 	fprintf (stderr, "bounding box margin: (%.06f * %.06f)", boxsize[0], -bbm);
       else
-	fprintf (stderr, "bounding box margin: %.06f", MAX(boxsize[0]*0.2, 2*shore_depth_tol));
+	fprintf(stderr, "bounding box margin: %.06f",
+	        fmax(boxsize[0] * 0.2, 2 * shore_depth_tol));
     }
     if (nrandom < 0) {
       double n1, n2, area2;
       area2 = (xmax[1] - xmin[1])*(xmax[0] - xmin[0]);
       n1 = (int) area2/(shore_depth_tol*shore_depth_tol);
       n2 = n*((int) area2/area);
-      nrandom = MAX(n1, n2);
+      nrandom = fmax(n1, n2);
     }
     srand(123);
     xran = gv_calloc((nrandom + 4) * dim2, sizeof(double));
@@ -1243,13 +1253,16 @@ static void make_map_internal(bool include_OK_points,
     }
   }
 
-  get_tri(n + nrandom, dim2, xcombined, &nt, &Tp, &E);
+  if (get_tri(n + nrandom, dim2, xcombined, &nt, &Tp, &E) != 0) {
+    goto done;
+  }
   get_polygons(n, nrandom, dim2, graph, grouping, nt, Tp, E, nverts, x_poly, poly_lines, polys, polys_groups,
 	       poly_point_map, country_graph);
 
-  free(xcombined);
   SparseMatrix_delete(E);
   free(Tp);
+done:
+  free(xcombined);
   free(xran);
   if (grouping != grouping0) free(grouping);
   if (x != x0) free(x);
@@ -1276,10 +1289,10 @@ static void get_boundingbox(int n, int dim, double *x, double *width, double *bb
   bbox[2] = bbox[3] = x[1];
   
   for (i = 0; i < n; i++){
-    bbox[0] = MIN(bbox[0], x[i*dim] - width[i*dim]);
-    bbox[1] = MAX(bbox[1], x[i*dim] + width[i*dim]);
-    bbox[2] = MIN(bbox[2], x[i*dim + 1] - width[i*dim+1]);
-    bbox[3] = MAX(bbox[3], x[i*dim + 1] + width[i*dim+1]);
+    bbox[0] = fmin(bbox[0], x[i * dim] - width[i * dim]);
+    bbox[1] = fmax(bbox[1], x[i * dim] + width[i * dim]);
+    bbox[2] = fmin(bbox[2], x[i * dim + 1] - width[i * dim + 1]);
+    bbox[3] = fmax(bbox[3], x[i * dim + 1] + width[i * dim + 1]);
   }
 }
 
