@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -11,12 +11,14 @@
 #include <windows.h>
 #include <io.h>
 #endif
+#include <assert.h>
 #include "viewport.h"
 #include "draw.h"
 #include <cgraph/alloc.h>
 #include <common/color.h>
 #include <glade/glade.h>
 #include "gui.h"
+#include <limits.h>
 #include "menucallbacks.h"
 #include <stddef.h>
 #include <stdbool.h>
@@ -126,39 +128,10 @@ void set_viewport_settings_from_template(ViewInfo *vi, Agraph_t *g) {
     vi->selectedNodeColor.B = (float)cl.u.RGBA[2];
     vi->selectedNodeColor.A = (float)
 	atof(get_attribute_value("selectednodecoloralpha", vi, g));
-    //selected edge are drawn with this color
-    colorxlate(get_attribute_value("selectededgecolor", vi, g), &cl,
-	       RGBA_DOUBLE);
-    vi->selectedEdgeColor.R = (float)cl.u.RGBA[0];
-    vi->selectedEdgeColor.G = (float)cl.u.RGBA[1];
-    vi->selectedEdgeColor.B = (float)cl.u.RGBA[2];
-    vi->selectedEdgeColor.A = (float)
-	atof(get_attribute_value("selectededgecoloralpha", vi, g));
 
-    colorxlate(get_attribute_value("highlightednodecolor", vi, g), &cl,
-	       RGBA_DOUBLE);
-    vi->highlightedNodeColor.R = (float)cl.u.RGBA[0];
-    vi->highlightedNodeColor.G = (float)cl.u.RGBA[1];
-    vi->highlightedNodeColor.B = (float)cl.u.RGBA[2];
-    vi->highlightedNodeColor.A = (float)
-	atof(get_attribute_value("highlightednodecoloralpha", vi, g));
-
-    buf = agget(g, "highlightededgecolor");
-    colorxlate(get_attribute_value("highlightededgecolor", vi, g), &cl,
-	       RGBA_DOUBLE);
-    vi->highlightedEdgeColor.R = (float)cl.u.RGBA[0];
-    vi->highlightedEdgeColor.G = (float)cl.u.RGBA[1];
-    vi->highlightedEdgeColor.B = (float)cl.u.RGBA[2];
-    vi->highlightedEdgeColor.A = (float)
-	atof(get_attribute_value("highlightededgecoloralpha", vi, g));
     vi->defaultnodealpha = (float)
 	atof(get_attribute_value("defaultnodealpha", vi, g));
 
-    vi->defaultedgealpha = (float)
-	atof(get_attribute_value("defaultedgealpha", vi, g));
-
-
-
     /*default line width */
     vi->LineWidth =
 	(float) atof(get_attribute_value("defaultlinewidth", vi, g));
@@ -319,7 +292,6 @@ void init_viewport(ViewInfo *vi) {
     vi->Topview->fisheyeParams.hier.dist2_limit = 1;
     vi->Topview->fisheyeParams.repos.width = (int)(vi->bdxRight - vi->bdxLeft);
     vi->Topview->fisheyeParams.repos.height = (int)(vi->bdyTop - vi->bdyBottom);
-    vi->Topview->fisheyeParams.repos.margin = 0;
     vi->Topview->fisheyeParams.repos.distortion = 1.0;
     /*create timer */
     vi->timer = g_timer_new();
@@ -333,7 +305,7 @@ void init_viewport(ViewInfo *vi) {
     g_timeout_add_full(G_PRIORITY_DEFAULT, 100u, gl_main_expose, NULL, NULL);
     vi->cameras = NULL;
     vi->camera_count = 0;
-    vi->active_camera = -1;
+    vi->active_camera = SIZE_MAX;
     set_viewport_settings_from_template(vi, vi->systemGraphs.def_attrs);
     vi->Topview->Graphdata.GraphFileName = NULL;
     vi->colschms = NULL;
@@ -469,8 +441,9 @@ int add_graph_to_viewport(Agraph_t * graph, char *id)
 	view->g[view->graphCount - 1] = graph;
 
 	gtk_combo_box_append_text(view->graphComboBox, id);
-	gtk_combo_box_set_active(view->graphComboBox,view->graphCount-1);
-	activate(view->graphCount - 1);
+	assert(view->graphCount <= INT_MAX);
+	gtk_combo_box_set_active(view->graphComboBox, (int)view->graphCount - 1);
+	activate((int)view->graphCount - 1);
 	return 1;
     } else {
 	return 0;
@@ -479,7 +452,7 @@ int add_graph_to_viewport(Agraph_t * graph, char *id)
 }
 void switch_graph(int graphId)
 {
-    if (graphId >= view->graphCount || graphId < 0)
+    if (graphId < 0 || (size_t)graphId >= view->graphCount)
 	return;			/*wrong entry */
     else
 	activate(graphId);
