@@ -1,9 +1,11 @@
+/// @file
+/// @ingroup common_utils
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -14,8 +16,8 @@
 #include "htmlparse.h"
 #include <common/htmllex.h>
 #include <cdt/cdt.h>
-#include <ctype.h>
 #include <cgraph/alloc.h>
+#include <cgraph/gv_ctype.h>
 #include <cgraph/startswith.h>
 #include <cgraph/strcasecmp.h>
 #include <cgraph/strview.h>
@@ -78,7 +80,7 @@ void htmlerror(const char *msg)
     if (state.error)
 	return;
     state.error = 1;
-    agerr(AGERR, "%s in line %d \n", msg, htmllineno());
+    agerrorf("%s in line %lu \n", msg, htmllineno());
     error_context();
 }
 
@@ -90,8 +92,7 @@ static void lexerror(const char *name)
 {
     state.tok = T_error;
     state.error = 1;
-    agerr(AGERR, "Unknown HTML element <%s> on line %d \n",
-	  name, htmllineno());
+    agerrorf("Unknown HTML element <%s> on line %lu \n", name, htmllineno());
 }
 
 typedef int (*attrFn) (void *, char *);
@@ -106,11 +107,11 @@ typedef struct {
 #define ISIZE (sizeof(attr_item))
 
 /* icmp:
- * Compare two attr_item. Used in bsearch
+ * Compare an attr_item. Used in bsearch
  */
-static int icmp(attr_item * i, attr_item * j)
-{
-    return strcasecmp(i->name, j->name);
+static int icmp(const void *name, const void *item) {
+  const attr_item *j = item;
+  return strcasecmp(name, j->name);
 }
 
 static int bgcolorfn(htmldata_t * p, char *v)
@@ -137,7 +138,7 @@ static int sidesfn(htmldata_t * p, char *v)
     char c;
 
     while ((c = *v++)) {
-	switch (tolower(c)) {
+	switch (gv_tolower(c)) {
 	case 'l' :
 	    flags |= BORDER_LEFT;
 	    break;
@@ -151,7 +152,7 @@ static int sidesfn(htmldata_t * p, char *v)
 	    flags |= BORDER_BOTTOM;
 	    break;
 	default :
-	    agerr(AGWARN, "Unrecognized character '%c' (%d) in sides attribute\n", c, c);
+	    agwarningf("Unrecognized character '%c' (%d) in sides attribute\n", c, c);
 	    break;
 	}
     }
@@ -187,7 +188,7 @@ static int stylefn(htmldata_t * p, char *v)
 	else if (strview_case_str_eq(tk,"DOTTED")) p->style |= DOTTED;
 	else if (strview_case_str_eq(tk,"DASHED")) p->style |= DASHED;
 	else {
-	    agerr(AGWARN, "Illegal value %.*s for STYLE - ignored\n", (int)tk.size,
+	    agwarningf("Illegal value %.*s for STYLE - ignored\n", (int)tk.size,
 	          tk.data);
 	    rv = 1;
 	}
@@ -221,13 +222,13 @@ static int doInt(char *v, char *s, int min, int max, long *ul)
     long b = strtol(v, &ep, 10);
 
     if (ep == v) {
-	agerr(AGWARN, "Improper %s value %s - ignored", s, v);
+	agwarningf("Improper %s value %s - ignored", s, v);
 	rv = 1;
     } else if (b > max) {
-	agerr(AGWARN, "%s value %s > %d - too large - ignored", s, v, max);
+	agwarningf("%s value %s > %d - too large - ignored", s, v, max);
 	rv = 1;
     } else if (b < min) {
-	agerr(AGWARN, "%s value %s < %d - too small - ignored", s, v, min);
+	agwarningf("%s value %s < %d - too small - ignored", s, v, min);
 	rv = 1;
     } else
 	*ul = b;
@@ -292,7 +293,7 @@ static int cellborderfn(htmltbl_t * p, char *v)
 static int columnsfn(htmltbl_t * p, char *v)
 {
     if (*v != '*') {
-	agerr(AGWARN, "Unknown value %s for COLUMNS - ignored\n", v);
+	agwarningf("Unknown value %s for COLUMNS - ignored\n", v);
 	return 1;
     }
     p->vrule = true;
@@ -302,7 +303,7 @@ static int columnsfn(htmltbl_t * p, char *v)
 static int rowsfn(htmltbl_t * p, char *v)
 {
     if (*v != '*') {
-	agerr(AGWARN, "Unknown value %s for ROWS - ignored\n", v);
+	agwarningf("Unknown value %s for ROWS - ignored\n", v);
 	return 1;
     }
     p->hrule = true;
@@ -315,7 +316,7 @@ static int fixedsizefn(htmldata_t * p, char *v)
     if (!strcasecmp(v, "TRUE"))
 	p->flags |= FIXED_FLAG;
     else if (strcasecmp(v, "FALSE")) {
-	agerr(AGWARN, "Illegal value %s for FIXEDSIZE - ignored\n", v);
+	agwarningf("Illegal value %s for FIXEDSIZE - ignored\n", v);
 	rv = 1;
     }
     return rv;
@@ -329,7 +330,7 @@ static int valignfn(htmldata_t * p, char *v)
     else if (!strcasecmp(v, "TOP"))
 	p->flags |= VALIGN_TOP;
     else if (strcasecmp(v, "MIDDLE")) {
-	agerr(AGWARN, "Illegal value %s for VALIGN - ignored\n", v);
+	agwarningf("Illegal value %s for VALIGN - ignored\n", v);
 	rv = 1;
     }
     return rv;
@@ -343,7 +344,7 @@ static int halignfn(htmldata_t * p, char *v)
     else if (!strcasecmp(v, "RIGHT"))
 	p->flags |= HALIGN_RIGHT;
     else if (strcasecmp(v, "CENTER")) {
-	agerr(AGWARN, "Illegal value %s for ALIGN - ignored\n", v);
+	agwarningf("Illegal value %s for ALIGN - ignored\n", v);
 	rv = 1;
     }
     return rv;
@@ -361,7 +362,7 @@ static int cell_halignfn(htmldata_t * p, char *v)
     else if (strcasecmp(v, "CENTER"))
 	rv = 1;
     if (rv)
-	agerr(AGWARN, "Illegal value %s for ALIGN in TD - ignored\n", v);
+	agwarningf("Illegal value %s for ALIGN in TD - ignored\n", v);
     return rv;
 }
 
@@ -375,7 +376,7 @@ static int balignfn(htmldata_t * p, char *v)
     else if (strcasecmp(v, "CENTER"))
 	rv = 1;
     if (rv)
-	agerr(AGWARN, "Illegal value %s for BALIGN in TD - ignored\n", v);
+	agwarningf("Illegal value %s for BALIGN in TD - ignored\n", v);
     return rv;
 }
 
@@ -406,7 +407,7 @@ static int rowspanfn(htmlcell_t * p, char *v)
     if (doInt(v, "ROWSPAN", 0, UINT16_MAX, &u))
 	return 1;
     if (u == 0) {
-	agerr(AGWARN, "ROWSPAN value cannot be 0 - ignored\n");
+	agwarningf("ROWSPAN value cannot be 0 - ignored\n");
 	return 1;
     }
     p->rowspan = (uint16_t)u;
@@ -420,7 +421,7 @@ static int colspanfn(htmlcell_t * p, char *v)
     if (doInt(v, "COLSPAN", 0, UINT16_MAX, &u))
 	return 1;
     if (u == 0) {
-	agerr(AGWARN, "COLSPAN value cannot be 0 - ignored\n");
+	agwarningf("COLSPAN value cannot be 0 - ignored\n");
 	return 1;
     }
     p->colspan = (uint16_t)u;
@@ -471,7 +472,7 @@ static int alignfn(int *p, char *v)
     else if (!strcasecmp(v, "CENTER"))
 	*p = 'n';
     else {
-	agerr(AGWARN, "Illegal value %s for ALIGN - ignored\n", v);
+	agwarningf("Illegal value %s for ALIGN - ignored\n", v);
 	rv = 1;
     }
     return rv;
@@ -551,22 +552,19 @@ static attr_item br_items[] = {
  * Name/value pairs are in array atts, which is null terminated.
  * s is the name of the HTML element being processed.
  */
-static void
-doAttrs(void *tp, attr_item * items, int nel, char **atts, char *s)
-{
+static void doAttrs(void *tp, attr_item *items, size_t nel, char **atts,
+                    char *s) {
     char *name;
     char *val;
     attr_item *ip;
-    attr_item key;
 
     while ((name = *atts++) != NULL) {
 	val = *atts++;
-	key.name = name;
-	ip = bsearch(&key, items, nel, ISIZE, (bcmpfn) icmp);
+	ip = bsearch(name, items, nel, ISIZE, icmp);
 	if (ip)
 	    state.warn |= ip->action(tp, val);
 	else {
-	    agerr(AGWARN, "Illegal attribute %s in %s - ignored\n", name,
+	    agwarningf("Illegal attribute %s in %s - ignored\n", name,
 		  s);
 	    state.warn = 1;
 	}
@@ -783,7 +781,7 @@ int initHTMLlexer(char *src, agxbuf * xb, htmlenv_t *env)
 #else
     static int first;
     if (!first) {
-	agerr(AGWARN,
+	agwarningf(
 	      "Not built with libexpat. Table formatting is not available.\n");
 	first++;
     }
@@ -835,7 +833,7 @@ static char *eatComment(char *p)
     if (*s) {
 	char *t = s - 2;
 	if (t < p || !startswith(t, "--")) {
-	    agerr(AGWARN, "Unclosed comment\n");
+	    agwarningf("Unclosed comment\n");
 	    state.warn = 1;
 	}
     }
@@ -858,7 +856,7 @@ static char *findNext(char *s, agxbuf* xb)
 	    while (*t && *t != '>')
 		t++;
 	if (*t != '>') {
-	    agerr(AGWARN, "Label closed before end of HTML element\n");
+	    agwarningf("Label closed before end of HTML element\n");
 	    state.warn = 1;
 	} else
 	    t++;
@@ -918,8 +916,7 @@ static void protect_rsqb(agxbuf *xb) {
 }
 #endif
 
-int htmllineno(void)
-{
+unsigned long htmllineno(void) {
 #ifdef HAVE_EXPAT
     return XML_GetCurrentLineNumber(state.parser);
 #else
@@ -1107,9 +1104,8 @@ int htmllex(void)
 	}
 	if (rv == XML_STATUS_ERROR) {
 	    if (!state.error) {
-		agerr(AGERR, "%s in line %d \n",
-		      XML_ErrorString(XML_GetErrorCode(state.parser)),
-		      htmllineno());
+		agerrorf("%s in line %lu \n",
+		      XML_ErrorString(XML_GetErrorCode(state.parser)), htmllineno());
 		error_context();
 		state.error = 1;
 		state.tok = T_error;
