@@ -1,14 +1,15 @@
+/// @file
+/// @ingroup common_render
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#include <ctype.h>
 #include <common/render.h>
 #include <common/htmltable.h>
 #include <errno.h>
@@ -16,12 +17,14 @@
 #include <xdot/xdot.h>
 #include <cgraph/agxbuf.h>
 #include <cgraph/alloc.h>
+#include <cgraph/gv_ctype.h>
 #include <cgraph/exit.h>
 #include <cgraph/startswith.h>
 #include <cgraph/strcasecmp.h>
 #include <cgraph/streq.h>
+#include <limits.h>
 #include <stdbool.h>
-#include <stddef.h>
+#include <stdlib.h>
 #include <string.h>
 
 static char *usageFmt =
@@ -65,8 +68,7 @@ static char *configItems = "\n\
                with available plugin information.  Needs write privilege.)\n\
  -?          - Print usage and exit\n";
 
-/* dotneato_usage:
- * Print usage information. If GvExitOnUsage is set, exit with
+/* Print usage information. If GvExitOnUsage is set, exit with
  * given exval, else return exval+1.
  */
 int dotneato_usage(int exval)
@@ -92,8 +94,7 @@ int dotneato_usage(int exval)
     return exval + 1;
 }
 
-/* getFlagOpt:
- * Look for flag parameter. idx is index of current argument.
+/* Look for flag parameter. idx is index of current argument.
  * We assume argv[*idx] has the form "-x..." If there are characters 
  * after the x, return
  * these, else if there are more arguments, return the next one,
@@ -117,8 +118,7 @@ static char *getFlagOpt(int argc, char **argv, int *idx)
     return 0;
 }
 
-/* dotneato_basename:
- * Partial implementation of real basename.
+/* Partial implementation of real basename.
  * Skip over any trailing slashes or backslashes; then
  * find next (back)slash moving left; return string to the right.
  * If no next slash is found, return the whole string.
@@ -151,11 +151,7 @@ static char *dotneato_basename(char *pathname) {
 #ifdef _WIN32
     /* On Windows, names are case-insensitive, so make name lower-case
      */
-    {
-	char c;
-	for (s = ret; (c = *s); s++)
-	    *s = tolower(c);
-    }
+    gv_tolower_str(ret);
 #endif
     return ret;
 }
@@ -212,8 +208,7 @@ graph_t *gvPluginsGraph(GVC_t *gvc)
     return P_graph;
 }
 
-/* dotneato_args_initialize"
- * Scan argv[] for allowed flags.
+/* Scan argv[] for allowed flags.
  * Return 0 on success; v+1 if calling function should call exit(v).
  * If -c is set, config file is created and we exit. 
  */
@@ -271,6 +266,9 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 	} else if (argv[i] &&
 	    (startswith(argv[i], "-?") || strcmp(argv[i], "--help") == 0)) {
 	    return dotneato_usage(0);
+	} else if (argv[i] && startswith(argv[i], "--filepath=")) {
+	    free(Gvfilepath);
+	    Gvfilepath = gv_strdup(argv[i] + strlen("--filepath="));
 	} else if (argv[i] && argv[i][0] == '-') {
 	    rest = &argv[i][2];
 	    switch (c = argv[i][1]) {
@@ -403,7 +401,7 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 		Y_invert = true;
 		break;
 	    default:
-		agerr(AGERR, "%s: option -%c unrecognized\n\n", gvc->common.cmdname,
+		agerrorf("%s: option -%c unrecognized\n\n", gvc->common.cmdname,
 			c);
 		return dotneato_usage(1);
 	    }
@@ -459,8 +457,7 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
     return 0;
 }
 
-/* getdoubles2ptf:
- * converts a graph attribute in inches to a pointf in points.
+/* converts a graph attribute in inches to a pointf in points.
  * If only one number is given, it is used for both x and y.
  * Returns true if the attribute ends in '!'.
  */
@@ -508,17 +505,17 @@ graph_t *gvNextInputGraph(GVC_t *gvc)
     static char *fn;
     static FILE *fp;
     static FILE *oldfp;
-    static int fidx, gidx;
+    static int gidx;
 
     while (!g) {
 	if (!fp) {
     	    if (!(fn = gvc->input_filenames[0])) {
-		if (fidx++ == 0)
+		if (gvc->fidx++ == 0)
 		    fp = stdin;
 	    }
 	    else {
-		while ((fn = gvc->input_filenames[fidx++]) && !(fp = fopen(fn, "r")))  {
-		    agerr(AGERR, "%s: can't open %s: %s\n", gvc->common.cmdname, fn, strerror(errno));
+		while ((fn = gvc->input_filenames[gvc->fidx++]) && !(fp = fopen(fn, "r")))  {
+		    agerrorf("%s: can't open %s: %s\n", gvc->common.cmdname, fn, strerror(errno));
 		    graphviz_errors++;
 		}
 	    }
@@ -542,8 +539,7 @@ graph_t *gvNextInputGraph(GVC_t *gvc)
     return g;
 }
 
-/* findCharset:
- * Check if the charset attribute is defined for the graph and, if
+/* Check if the charset attribute is defined for the graph and, if
  * so, return the corresponding internal value. If undefined, return
  * CHAR_UTF8
  */
@@ -565,13 +561,11 @@ static unsigned char findCharset(graph_t *g) {
     if (!strcasecmp(p,"utf-8")
 	|| !strcasecmp(p,"utf8"))
       return CHAR_UTF8;
-    agerr(AGWARN, "Unsupported charset \"%s\" - assuming utf-8\n", p);
+    agwarningf("Unsupported charset \"%s\" - assuming utf-8\n", p);
     return CHAR_UTF8;
 }
 
-/* setRatio:
- * Checks "ratio" attribute, if any, and sets enum type.
- */
+/// Checks "ratio" attribute, if any, and sets enum type.
 static void setRatio(graph_t * g)
 {
     char *p;
@@ -632,6 +626,9 @@ void graph_init(graph_t * g, bool use_rankdir)
 
     if (!HTTPServerEnVar) {
 	Gvimagepath = agget (g, "imagepath");
+	if (!Gvimagepath) {
+	    Gvimagepath = Gvfilepath;
+	}
     }
 
     GD_drawing(g)->quantum =
@@ -676,7 +673,13 @@ void graph_init(graph_t * g, bool use_rankdir)
 	xf = DEFAULT_RANKSEP;
     GD_ranksep(g) = POINTS(xf);
 
-    GD_showboxes(g) = late_int(g, agfindgraphattr(g, "showboxes"), 0, 0);
+    {
+	int showboxes = late_int(g, agfindgraphattr(g, "showboxes"), 0, 0);
+	if (showboxes > UCHAR_MAX) {
+	    showboxes = UCHAR_MAX;
+	}
+	GD_showboxes(g) = (unsigned char)showboxes;
+    }
     p = late_string(g, agfindgraphattr(g, "fontnames"), NULL);
     GD_fontnames(g) = maptoken(p, fontnamenames, fontnamecodes);
 
@@ -802,10 +805,7 @@ void graph_cleanup(graph_t *g)
     agclean(g, AGRAPH,"Agraphinfo_t");
 }
 
-/* charsetToStr:
- * Given an internal charset value, return a canonical string
- * representation.
- */
+/// Given an internal charset value, return a canonical string representation.
 char*
 charsetToStr (int c)
 {
@@ -822,17 +822,14 @@ charsetToStr (int c)
 	s = "BIG-5";
 	break;
    default :
-	agerr(AGERR, "Unsupported charset value %d\n", c);
+	agerrorf("Unsupported charset value %d\n", c);
 	s = "UTF-8";
 	break;
    }
    return s;
 }
 
-/* do_graph_label:
- * Set characteristics of graph label if it exists.
- * 
- */
+/// Set characteristics of graph label if it exists.
 void do_graph_label(graph_t * sg)
 {
     char *str, *pos, *just;
