@@ -1,9 +1,11 @@
+/// @file
+/// @ingroup common_utils
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -27,9 +29,7 @@ static void freeIntItem(intitem *obj, Dtdisc_t *disc) {
     free (obj);
 }
 
-static int
-cmpid(Dt_t* d, int* key1, int* key2, Dtdisc_t* disc)
-{
+static int cmpid(Dt_t *d, size_t *key1, size_t *key2, Dtdisc_t *disc) {
   (void)d;
   (void)disc;
 
@@ -53,18 +53,14 @@ openIntSet (void)
     return dtopen(&intSetDisc,Dtoset);
 }
 
-void 
-addIntSet (Dt_t* is, int v)
-{
+void addIntSet(Dt_t *is, size_t v) {
     intitem obj;
 
     obj.id = v;
     dtinsert(is, &obj);
 }
 
-int 
-inIntSet (Dt_t* is, int v)
-{
+int inIntSet(Dt_t *is, size_t v) {
     return (dtmatch (is, &v) != 0);
 }
 
