@@ -1,9 +1,12 @@
+/// @file
+/// @brief  [edge splines](https://graphviz.org/docs/attrs/splines/)
+/// @ingroup common_render
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -215,7 +218,8 @@ bezier *new_spline(edge_t *e, size_t sz) {
 	e = ED_to_orig(e);
     if (ED_spl(e) == NULL)
 	ED_spl(e) = gv_alloc(sizeof(splines));
-    ED_spl(e)->list = ALLOC(ED_spl(e)->size + 1, ED_spl(e)->list, bezier);
+    ED_spl(e)->list = gv_recalloc(ED_spl(e)->list, ED_spl(e)->size,
+                                  ED_spl(e)->size + 1, sizeof(bezier));
     rv = &(ED_spl(e)->list[ED_spl(e)->size++]);
     rv->list = gv_calloc(sz, sizeof(pointf));
     rv->size = sz;
@@ -813,21 +817,19 @@ else
   return pair_a[tail_i][head_i];
 }
 
-
-static void selfBottom (edge_t* edges[], int ind, int cnt,
-	double sizex, double stepy, splineInfo* sinfo)
-{
+static void selfBottom(edge_t *edges[], size_t ind, size_t cnt, double sizex,
+                       double stepy, splineInfo *sinfo) {
     pointf tp, hp, np;
     node_t *n;
     edge_t *e;
-    int i, sgn, point_pair;
+    int sgn, point_pair;
     double hy, ty, stepx, dx, dy, height;
     pointf points[1000];
 
     e = edges[ind];
     n = agtail(e);
 
-    stepx = fmax(sizex / 2.0 / cnt, 2.0);
+    stepx = fmax(sizex / 2.0 / (double)cnt, 2.0);
     np = ND_coord(n);
     tp = ED_tail_port(e).p;
     tp.x += np.x;
@@ -850,7 +852,7 @@ static void selfBottom (edge_t* edges[], int ind, int cnt,
     }
     ty = fmin(dy, 3 * (tp.y + dy - np.y));
     hy = fmin(dy, 3 * (hp.y + dy - np.y));
-    for (i = 0; i < cnt; i++) {
+    for (size_t i = 0; i < cnt; i++) {
         e = edges[ind++];
         dy += stepy;
         ty += stepy;
@@ -884,12 +886,9 @@ static void selfBottom (edge_t* edges[], int ind, int cnt,
     }
 }
 
-
-static void
-selfTop (edge_t* edges[], int ind, int cnt, double sizex, double stepy,
-           splineInfo* sinfo)
-{
-    int i, sgn, point_pair;
+static void selfTop(edge_t *edges[], size_t ind, size_t cnt, double sizex,
+                    double stepy, splineInfo *sinfo) {
+    int sgn, point_pair;
     double hy, ty,  stepx, dx, dy, height;
     pointf tp, hp, np;
     node_t *n;
@@ -899,7 +898,7 @@ selfTop (edge_t* edges[], int ind, int cnt, double sizex, double stepy,
     e = edges[ind];
     n = agtail(e);
 
-    stepx = fmax(sizex / 2.0 / cnt, 2.0);
+    stepx = fmax(sizex / 2.0 / (double)cnt, 2.0);
     np = ND_coord(n);
     tp = ED_tail_port(e).p;
     tp.x += np.x;
@@ -959,7 +958,7 @@ selfTop (edge_t* edges[], int ind, int cnt, double sizex, double stepy,
     }
     ty = fmin(dy, 3 * (np.y + dy - tp.y));
     hy = fmin(dy, 3 * (np.y + dy - hp.y));
-    for (i = 0; i < cnt; i++) {
+    for (size_t i = 0; i < cnt; i++) {
         e = edges[ind++];
         dy += stepy;
         ty += stepy;
@@ -993,11 +992,9 @@ selfTop (edge_t* edges[], int ind, int cnt, double sizex, double stepy,
     }
 }
 
-static void
-selfRight (edge_t* edges[], int ind, int cnt, double stepx, double sizey,
-           splineInfo* sinfo)
-{
-    int i, sgn, point_pair;
+static void selfRight(edge_t *edges[], size_t ind, size_t cnt, double stepx,
+                      double sizey, splineInfo *sinfo) {
+    int sgn, point_pair;
     double hx, tx, stepy, dx, dy, width;
     pointf tp, hp, np;
     node_t *n;
@@ -1007,7 +1004,7 @@ selfRight (edge_t* edges[], int ind, int cnt, double stepx, double sizey,
     e = edges[ind];
     n = agtail(e);
 
-    stepy = fmax(sizey / 2.0 / cnt, 2.0);
+    stepy = fmax(sizey / 2.0 / (double)cnt, 2.0);
     np = ND_coord(n);
     tp = ED_tail_port(e).p;
     tp.x += np.x;
@@ -1031,7 +1028,7 @@ selfRight (edge_t* edges[], int ind, int cnt, double stepx, double sizey,
     }
     tx = fmin(dx, 3 * (np.x + dx - tp.x));
     hx = fmin(dx, 3 * (np.x + dx - hp.x));
-    for (i = 0; i < cnt; i++) {
+    for (size_t i = 0; i < cnt; i++) {
         e = edges[ind++];
         dx += stepx;
         tx += stepx;
@@ -1065,11 +1062,9 @@ selfRight (edge_t* edges[], int ind, int cnt, double stepx, double sizey,
     }
 }
 
-static void
-selfLeft (edge_t* edges[], int ind, int cnt, double stepx, double sizey,
-          splineInfo* sinfo)
-{
-    int i, sgn,point_pair;
+static void selfLeft(edge_t *edges[], size_t ind, size_t cnt, double stepx,
+                     double sizey, splineInfo *sinfo) {
+    int sgn,point_pair;
     double hx, tx, stepy, dx, dy, width;
     pointf tp, hp, np;
     node_t *n;
@@ -1079,7 +1074,7 @@ selfLeft (edge_t* edges[], int ind, int cnt, double stepx, double sizey,
     e = edges[ind];
     n = agtail(e);
 
-    stepy = fmax(sizey / 2.0 / cnt, 2.0);
+    stepy = fmax(sizey / 2.0 / (double)cnt, 2.0);
     np = ND_coord(n);
     tp = ED_tail_port(e).p;
     tp.x += np.x;
@@ -1106,7 +1101,7 @@ selfLeft (edge_t* edges[], int ind, int cnt, double stepx, double sizey,
     }
     tx = fmin(dx, 3 * (tp.x + dx - np.x));
     hx = fmin(dx, 3 * (hp.x + dx - np.x));
-    for (i = 0; i < cnt; i++) {
+    for (size_t i = 0; i < cnt; i++) {
         e = edges[ind++];
         dx += stepx;
         tx += stepx;
@@ -1176,13 +1171,9 @@ double selfRightSpace(edge_t *e) {
  * FIX: With this bias, labels tend to be placed on top of each other.
  * Perhaps for self-edges, the label should be centered.
  */
-void
-makeSelfEdge(edge_t * edges[], int ind, int cnt, double sizex,
-	     double sizey, splineInfo * sinfo)
-{
-    edge_t *e;
-
-    e = edges[ind];
+void makeSelfEdge(edge_t *edges[], size_t ind, size_t cnt, double sizex,
+                  double sizey, splineInfo * sinfo) {
+    edge_t *e = edges[ind];
 
     /* self edge without ports or
      * self edge with all ports inside, on the right, or at most 1 on top
@@ -1406,7 +1397,7 @@ splines *getsplinepoints(edge_t * e)
     for (le = e; !(sp = ED_spl(le)) && ED_edge_type(le) != NORMAL;
 	 le = ED_to_orig(le));
     if (sp == NULL)
-	agerr (AGERR, "getsplinepoints: no spline points available for edge (%s,%s)\n",
+	agerrorf("getsplinepoints: no spline points available for edge (%s,%s)\n",
 	    agnameof(agtail(e)), agnameof(aghead(e)));
     return sp;
 }
