@@ -1,9 +1,11 @@
+/// @file
+/// @ingroup common_utils
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -12,6 +14,7 @@
 #include <cgraph/alloc.h>
 #include <cgraph/agxbuf.h>
 #include <cgraph/gv_ctype.h>
+#include <cgraph/gv_math.h>
 #include <cgraph/strview.h>
 #include <cgraph/tokenize.h>
 #include <common/htmltable.h>
@@ -27,45 +30,6 @@
 #include <stdint.h>
 #include <unistd.h>
 
-/*
- *  a queue of nodes
- */
-nodequeue *new_queue(size_t sz) {
-    nodequeue *q = gv_alloc(sizeof(nodequeue));
-
-    if (sz <= 1)
-	sz = 2;
-    q->head = q->tail = q->store = gv_calloc(sz, sizeof(node_t*));
-    q->limit = q->store + sz;
-    return q;
-}
-
-void free_queue(nodequeue * q)
-{
-    free(q->store);
-    free(q);
-}
-
-void enqueue(nodequeue * q, node_t * n)
-{
-    *q->tail++ = n;
-    if (q->tail >= q->limit)
-	q->tail = q->store;
-}
-
-node_t *dequeue(nodequeue * q)
-{
-    node_t *n;
-    if (q->head == q->tail)
-	n = NULL;
-    else {
-	n = *q->head++;
-	if (q->head >= q->limit)
-	    q->head = q->store;
-    }
-    return n;
-}
-
 int late_int(void *obj, attrsym_t *attr, int defaultValue, int minimum) {
     if (attr == NULL)
         return defaultValue;
@@ -78,7 +42,7 @@ int late_int(void *obj, attrsym_t *attr, int defaultValue, int minimum) {
         return defaultValue; /* invalid int format */
     if (rv < minimum)
         return minimum;
-    else return (int)rv;
+    return (int)rv;
 }
 
 double late_double(void *obj, attrsym_t *attr, double defaultValue,
@@ -94,11 +58,10 @@ double late_double(void *obj, attrsym_t *attr, double defaultValue,
         return defaultValue; /* invalid double format */
     if (rv < minimum)
         return minimum;
-    else return rv;
+    return rv;
 }
 
-/* get_inputscale:
- * Return value for PSinputscale. If this is > 0, it has been set on the
+/** Return value for PSinputscale. If this is > 0, it has been set on the
  * command line and this value is used.
  * Otherwise, we check the graph's inputscale attribute. If this is not set
  * or has a bad value, we return -1.
@@ -108,8 +71,8 @@ double late_double(void *obj, attrsym_t *attr, double defaultValue,
 double get_inputscale(graph_t *g) {
     if (PSinputscale > 0) return PSinputscale;  /* command line flag prevails */
     double d = late_double(g, agfindgraphattr(g, "inputscale"), -1, 0);
-    if (d == 0) return POINTS_PER_INCH;
-    else return d;
+    if (is_exactly_zero(d)) return POINTS_PER_INCH;
+    return d;
 }
 
 char *late_string(void *obj, attrsym_t *attr, char *defaultValue) {
@@ -198,7 +161,6 @@ pointf coord(node_t * n)
 #define W_DEGREE 5
 
 /*
- *  Bezier :
  *	Evaluate a Bezier curve at a particular parameter value
  *      Fill in control points for resulting sub-curves if "Left" and
  *	"Right" are non-null.
@@ -242,8 +204,7 @@ edge_t *debug_getedge(graph_t * g, char *s0, char *s1)
     n1 = agfindnode(g, s1);
     if (n0 && n1)
 	return agfindedge(g, n0, n1);
-    else
-	return NULL;
+    return NULL;
 }
 Agraphinfo_t* GD_info(graph_t * g) { return ((Agraphinfo_t*)AGDATA(g));}
 Agnodeinfo_t* ND_info(node_t * n) { return ((Agnodeinfo_t*)AGDATA(n));}
@@ -306,7 +267,7 @@ const char *safefile(const char *filename)
 
     if (HTTPServerEnVar) {   /* If used as a server */
 	if (onetime) {
-	    agerr(AGWARN,
+	    agwarningf(
 		      "file loading is disabled because the environment contains SERVER_NAME=\"%s\"\n",
 		      HTTPServerEnVar);
 	    onetime = false;
@@ -314,6 +275,24 @@ const char *safefile(const char *filename)
 	return NULL;
     }
 
+    if (Gvfilepath != NULL) {
+	if (pathlist == NULL) {
+	    free(dirs);
+	    pathlist = Gvfilepath;
+	    dirs = mkDirlist(pathlist);
+	}
+
+	const char *str = filename;
+	for (const char *sep = "/\\:"; *sep != '\0'; ++sep) {
+	    const char *p = strrchr(str, *sep);
+	    if (p != NULL) {
+	        str = ++p;
+	    }
+	}
+
+	return findPath(dirs, str);
+    }
+
     if (pathlist != Gvimagepath) {
 	free (dirs);
 	dirs = NULL;
@@ -351,8 +330,7 @@ bool mapBool(const char *p, bool defaultValue) {
 	return true;
     if (gv_isdigit(*p))
 	return atoi(p) != 0;
-    else
-        return defaultValue;
+    return defaultValue;
 }
 
 bool mapbool(const char *p)
@@ -464,7 +442,10 @@ void common_init_node(node_t * n)
 	GD_has_labels(agraphof(n)) |= NODE_XLABEL;
     }
 
-    ND_showboxes(n) = late_int(n, N_showboxes, 0, 0);
+    {
+	const int showboxes = imin(late_int(n, N_showboxes, 0, 0), UCHAR_MAX);
+	ND_showboxes(n) = (unsigned char)showboxes;
+    }
     ND_shape(n)->fns->initfn(n);
 }
 
@@ -485,10 +466,7 @@ initFontLabelEdgeAttr(edge_t * e, struct fontinfo *fi,
     lfi->fontcolor = late_nnstring(e, E_labelfontcolor, fi->fontcolor);
 }
 
-/* noClip:
- * Return true if head/tail end of edge should not be clipped
- * to node.
- */
+/// Return true if head/tail end of edge should not be clipped to node.
 static bool
 noClip(edge_t *e, attrsym_t* sym)
 {
@@ -503,8 +481,6 @@ noClip(edge_t *e, attrsym_t* sym)
     return rv;
 }
 
-/*chkPort:
- */
 static port
 chkPort (port (*pf)(node_t*, char*, char*), node_t* n, char* s)
 {
@@ -526,23 +502,23 @@ chkPort (port (*pf)(node_t*, char*, char*), node_t* n, char* s)
 }
 
 /* return true if edge has label */
-int common_init_edge(edge_t * e)
-{
+void common_init_edge(edge_t *e) {
     char *str;
-    int r = 0;
     struct fontinfo fi;
     struct fontinfo lfi;
     graph_t *sg = agraphof(agtail(e));
 
     fi.fontname = NULL;
     lfi.fontname = NULL;
-    if (E_label && (str = agxget(e, E_label)) && str[0]) {
-	r = 1;
+    if (E_label && (str = agxget(e, E_label))) {
 	initFontEdgeAttr(e, &fi);
-	ED_label(e) = make_label(e, str, aghtmlstr(str) ? LT_HTML : LT_NONE,
+	const int kind = !streq(str, "") && aghtmlstr(str) ? LT_HTML : LT_NONE;
+	ED_label(e) = make_label(e, str, kind,
 				fi.fontsize, fi.fontname, fi.fontcolor);
-	GD_has_labels(sg) |= EDGE_LABEL;
-	ED_label_ontop(e) = mapbool(late_string(e, E_label_float, "false"));
+	if (!streq(str, "")) {
+	    GD_has_labels(sg) |= EDGE_LABEL;
+	    ED_label_ontop(e) = mapbool(late_string(e, E_label_float, "false"));
+	}
     }
 
     if (E_xlabel && (str = agxget(e, E_xlabel)) && str[0]) {
@@ -587,12 +563,8 @@ int common_init_edge(edge_t * e)
     ED_head_port(e) = chkPort(ND_shape(aghead(e))->fns->portfn, aghead(e), str);
     if (noClip(e, E_headclip))
 	ED_head_port(e).clip = false;
-
-    return r;
 }
 
-/* addLabelBB:
- */
 static boxf addLabelBB(boxf bb, textlabel_t * lp, bool flipxy)
 {
     double width, height;
@@ -624,8 +596,7 @@ static boxf addLabelBB(boxf bb, textlabel_t * lp, bool flipxy)
     return bb;
 }
 
-/* polyBB:
- * Compute the bounding box of a polygon.
+/** Compute the bounding box of a polygon.
  * We only need to use the outer periphery.
  */
 boxf
@@ -646,8 +617,7 @@ polyBB (polygon_t* poly)
     return bb;
 }
 
-/* updateBB:
- * Reset graph's bounding box to include bounding box of the given label.
+/** Reset graph's bounding box to include bounding box of the given label.
  * Assume the label's position has been set.
  */
 void updateBB(graph_t * g, textlabel_t * lp)
@@ -655,8 +625,7 @@ void updateBB(graph_t * g, textlabel_t * lp)
     GD_bb(g) = addLabelBB(GD_bb(g), lp, GD_flip(g));
 }
 
-/* compute_bb:
- * Compute bounding box of g using nodes, splines, and clusters.
+/** Compute bounding box of g using nodes, splines, and clusters.
  * Assumes bb of clusters already computed.
  * store in GD_bb.
  */
@@ -725,11 +694,10 @@ void compute_bb(graph_t * g)
 bool is_a_cluster (Agraph_t* g)
 {
   return g == g->root || !strncasecmp(agnameof(g), "cluster", 7) ||
-         mapBool(agget(g, "cluster"), false);
+         mapbool(agget(g, "cluster"));
 }
 
-/* setAttr:
- * Sets object's name attribute to the given value.
+/** Sets object's name attribute to the given value.
  * Creates the attribute if not already set.
  */
 Agsym_t *setAttr(graph_t * g, void *obj, char *name, char *value,
@@ -752,8 +720,7 @@ Agsym_t *setAttr(graph_t * g, void *obj, char *name, char *value,
     return ap;
 }
 
-/* clustNode:
- * Generate a special cluster node representing the end node
+/** Generate a special cluster node representing the end node
  * of an edge to the cluster cg. n is a node whose name is the same
  * as the cluster cg. clg is the subgraph of all of
  * the original nodes, which will be deleted later.
@@ -795,18 +762,15 @@ static int cmpItem(Dt_t * d, void *p1[], void *p2[], Dtdisc_t * disc)
 
     if (p1[0] < p2[0])
 	return -1;
-    else if (p1[0] > p2[0])
+    if (p1[0] > p2[0])
 	return 1;
-    else if (p1[1] < p2[1])
+    if (p1[1] < p2[1])
 	return -1;
-    else if (p1[1] > p2[1])
+    if (p1[1] > p2[1])
 	return 1;
-    else
-	return 0;
+    return 0;
 }
 
-/* newItem:
- */
 static void *newItem(item *objp, Dtdisc_t *disc) {
     item *newp = gv_alloc(sizeof(item));
 
@@ -819,8 +783,6 @@ static void *newItem(item *objp, Dtdisc_t *disc) {
     return newp;
 }
 
-/* freeItem:
- */
 static void freeItem(item *obj, Dtdisc_t *disc) {
     (void)disc;
     free(obj);
@@ -835,9 +797,7 @@ static Dtdisc_t mapDisc = {
     .comparf = (Dtcompar_f)cmpItem,
 };
 
-/* cloneEdge:
- * Make a copy of e in e's graph but using ct and ch as nodes
- */
+/// Make a copy of e in e's graph but using ct and ch as nodes
 static edge_t *cloneEdge(edge_t * e, node_t * ct, node_t * ch)
 {
     graph_t *g = agraphof(ct);
@@ -849,8 +809,6 @@ static edge_t *cloneEdge(edge_t * e, node_t * ct, node_t * ch)
     return ce;
 }
 
-/* insertEdge:
- */
 static void insertEdge(Dt_t * map, void *t, void *h, edge_t * e)
 {
     item dummy;
@@ -868,10 +826,7 @@ static void insertEdge(Dt_t * map, void *t, void *h, edge_t * e)
     dtinsert(map, &dummy);
 }
 
-/* mapEdge:
- * Check if we already have cluster edge corresponding to t->h,
- * and return it.
- */
+/// Check if we already have cluster edge corresponding to t->h, and return it.
 static item *mapEdge(Dt_t * map, edge_t * e)
 {
     void *key[2];
@@ -881,8 +836,7 @@ static item *mapEdge(Dt_t * map, edge_t * e)
     return dtmatch(map, &key);
 }
 
-/* checkCompound:
- * If endpoint names a cluster, mark for temporary deletion and create
+/** If endpoint names a cluster, mark for temporary deletion and create
  * special node and insert into cluster. Then clone the edge. Real edge
  * will be deleted when we delete the original node.
  * Invariant: new edge has same sense as old. That is, given t->h with
@@ -917,7 +871,7 @@ checkCompound(edge_t * e, graph_t * clg, agxbuf * xb, Dt_t * map, Dt_t* cmap)
     if (!tg && !hg)
 	return 0;
     if (tg == hg) {
-	agerr(AGWARN, "cluster cycle %s -- %s not supported\n", agnameof(t),
+	agwarningf("cluster cycle %s -- %s not supported\n", agnameof(t),
 	      agnameof(t));
 	return 0;
     }
@@ -930,12 +884,12 @@ checkCompound(edge_t * e, graph_t * clg, agxbuf * xb, Dt_t * map, Dt_t* cmap)
     if (hg) {
 	if (tg) {
 	    if (agcontains(hg, tg)) {
-		agerr(AGWARN, "tail cluster %s inside head cluster %s\n",
+		agwarningf("tail cluster %s inside head cluster %s\n",
 		      agnameof(tg), agnameof(hg));
 		return 0;
 	    }
 	    if (agcontains(tg, hg)) {
-		agerr(AGWARN, "head cluster %s inside tail cluster %s\n",
+		agwarningf("head cluster %s inside tail cluster %s\n",
 		      agnameof(hg),agnameof(tg));
 		return 0;
 	    }
@@ -945,7 +899,7 @@ checkCompound(edge_t * e, graph_t * clg, agxbuf * xb, Dt_t * map, Dt_t* cmap)
 	    insertEdge(map, t, h, ce);
 	} else {
 	    if (agcontains(hg, t)) {
-		agerr(AGWARN, "tail node %s inside head cluster %s\n",
+		agwarningf("tail node %s inside head cluster %s\n",
 		      agnameof(t), agnameof(hg));
 		return 0;
 	    }
@@ -955,7 +909,7 @@ checkCompound(edge_t * e, graph_t * clg, agxbuf * xb, Dt_t * map, Dt_t* cmap)
 	}
     } else {
 	if (agcontains(tg, h)) {
-	    agerr(AGWARN, "head node %s inside tail cluster %s\n", agnameof(h),
+	    agwarningf("head node %s inside tail cluster %s\n", agnameof(h),
 		  agnameof(tg));
 	    return 0;
 	}
@@ -977,12 +931,10 @@ num_clust_edges(graph_t * g)
     cl_edge_t* cl_info = (cl_edge_t*)HAS_CLUST_EDGE(g);
     if (cl_info)
 	return cl_info->n_cluster_edges;
-    else
-	return 0;
+    return 0;
 }
 
-/* processClusterEdges:
- * Look for cluster edges. Replace cluster edge endpoints
+/** Look for cluster edges. Replace cluster edge endpoints
  * corresponding to a cluster with special cluster nodes.
  * Delete original nodes.
  * If cluster edges are found, a cl_edge_t record will be
@@ -1023,8 +975,7 @@ void processClusterEdges(graph_t * g)
     dtclose(cmap);
 }
 
-/* mapN:
- * Convert cluster nodes back to ordinary nodes
+/** Convert cluster nodes back to ordinary nodes
  * If n is already ordinary, return it.
  * Otherwise, we know node's name is "__i:xxx"
  * where i is some number and xxx is the nodes's original name.
@@ -1084,8 +1035,7 @@ static void undoCompound(edge_t * e, graph_t * clg)
     gv_cleanup_edge(e);
 }
 
-/* undoClusterEdges:
- * Replace cluster nodes with originals. Make sure original has
+/** Replace cluster nodes with originals. Make sure original has
  * no attributes. Replace original edges. Delete cluster nodes,
  * which will also delete cluster edges.
  */
@@ -1120,8 +1070,7 @@ void undoClusterEdges(graph_t * g)
     agclose(clg);
 }
 
-/* safe_dcl:
- * Find the attribute belonging to graph g for objects like obj
+/** Find the attribute belonging to graph g for objects like obj
  * with given name. If one does not exist, create it with the
  * default value defaultValue.
  */
@@ -1133,20 +1082,19 @@ attrsym_t *safe_dcl(graph_t *g, int obj_kind, char *name, char *defaultValue) {
 }
 
 static int comp_entities(const void *e1, const void *e2) {
-  return strcmp(((const struct entities_s *)e1)->name, ((const struct entities_s *)e2)->name);
+  return strcmp(e1, ((const struct entities_s *)e2)->name);
 }
 
 #define MAXENTLEN 8
 
-/* scanEntity:
- * Scan non-numeric entity, convert to &#...; form and store in xbuf.
+/** Scan non-numeric entity, convert to &#...; form and store in xbuf.
  * t points to first char after '&'. Return after final semicolon.
  * If unknown, we return t and let libexpat flag the error.
  *     */
 char* scanEntity (char* t, agxbuf* xb)
 {
     char*  endp = strchr (t, ';');
-    struct entities_s key, *res;
+    struct entities_s *res;
     size_t len;
     char   buf[MAXENTLEN+1];
 
@@ -1155,17 +1103,14 @@ char* scanEntity (char* t, agxbuf* xb)
     if ((len = (size_t)(endp - t)) > MAXENTLEN || len < 2) return t;
     strncpy (buf, t, len);
     buf[len] = '\0';
-    key.name =  buf;
-    res = bsearch(&key, entities, NR_OF_ENTITIES,
+    res = bsearch(buf, entities, NR_OF_ENTITIES,
         sizeof(entities[0]), comp_entities);
     if (!res) return t;
     agxbprint(xb, "#%d;", res->value);
     return endp + 1;
 }
 
-
-/* htmlEntity:
- * Check for an HTML entity for a special character.
+/** Check for an HTML entity for a special character.
  * Assume *s points to first byte after '&'.
  * If successful, return the corresponding value and update s to
  * point after the terminating ';'.
@@ -1175,7 +1120,7 @@ static int
 htmlEntity (char** s)
 {
     char *p;
-    struct entities_s key, *res;
+    struct entities_s *res;
     char entity_name_buf[ENTITY_NAME_LENGTH_MAX+1];
     unsigned char* str = *(unsigned char**)s;
     unsigned int byte;
@@ -1215,14 +1160,14 @@ htmlEntity (char** s)
 	}
     }
     else {
-	key.name = p = entity_name_buf;
+	p = entity_name_buf;
 	for (i = 0; i <  ENTITY_NAME_LENGTH_MAX; i++) {
 	    byte = *(str + i);
 	    if (byte == '\0') break;
 	    if (byte == ';') {
 		*p++ = '\0';
-		res = bsearch(&key, entities, NR_OF_ENTITIES,
-		    sizeof(entities[0]), *comp_entities);
+		res = bsearch(entity_name_buf, entities, NR_OF_ENTITIES,
+		    sizeof(entities[0]), comp_entities);
 		if (res) {
 		    n = res->value;
 		    str += i+1;
@@ -1254,8 +1199,7 @@ cvtAndAppend (unsigned char c, agxbuf* xb)
     return c;
 }
 
-/* htmlEntityUTF8:
- * substitute html entities like: &#123; and: &amp; with the UTF8 equivalents
+/** substitute html entities like: &#123; and: &amp; with the UTF8 equivalents
  * check for invalid utf8. If found, treat a single byte as Latin-1, convert it to
  * utf8 and warn the user.
  */
@@ -1294,7 +1238,7 @@ char* htmlEntityUTF8 (char* s, graph_t* g)
         else {
             uc = -1;
             if (!warned) {
-                agerr(AGWARN, "UTF8 codes > 4 bytes are not currently supported (graph %s) - treated as Latin-1. Perhaps \"-Gcharset=latin1\" is needed?\n", agnameof(g));
+                agwarningf("UTF8 codes > 4 bytes are not currently supported (graph %s) - treated as Latin-1. Perhaps \"-Gcharset=latin1\" is needed?\n", agnameof(g));
                 warned = true;
             }
             c = cvtAndAppend (c, &xb);
@@ -1326,7 +1270,7 @@ char* htmlEntityUTF8 (char* s, graph_t* g)
                 }
                 else {
 		            if (!warned) {
-		                agerr(AGWARN, "Invalid %d-byte UTF8 found in input of graph %s - treated as Latin-1. Perhaps \"-Gcharset=latin1\" is needed?\n", uc + 1, agnameof(g));
+		                agwarningf("Invalid %d-byte UTF8 found in input of graph %s - treated as Latin-1. Perhaps \"-Gcharset=latin1\" is needed?\n", uc + 1, agnameof(g));
 		                warned = true;
 		            }
 		            c = cvtAndAppend (c, &xb);
@@ -1337,11 +1281,7 @@ char* htmlEntityUTF8 (char* s, graph_t* g)
     return agxbdisown(&xb);
 }
 
-/* latin1ToUTF8:
- * Converts string from Latin1 encoding to utf8
- * Also translates HTML entities.
- *
- */
+/// Converts string from Latin1 encoding to utf8. Also translates HTML entities.
 char* latin1ToUTF8 (char* s)
 {
     agxbuf xb = {0};
@@ -1370,8 +1310,7 @@ char* latin1ToUTF8 (char* s)
     return agxbdisown(&xb);
 }
 
-/* utf8ToLatin1:
- * Converts string from utf8 encoding to Latin1
+/** Converts string from utf8 encoding to Latin1
  * Note that it does not attempt to reproduce HTML entities.
  * We assume the input string comes from latin1ToUTF8.
  */
@@ -1458,9 +1397,7 @@ bool overlap_edge(edge_t *e, boxf b)
     return false;
 }
 
-/* edgeType:
- * Convert string to edge type.
- */
+/// Convert string to edge type.
 static int edgeType(const char *s, int defaultValue) {
     if (s == NULL || strcmp(s, "") == 0) {
         return defaultValue;
@@ -1494,12 +1431,11 @@ static int edgeType(const char *s, int defaultValue) {
 	return EDGETYPE_SPLINE;
     }
 
-    agerr(AGWARN, "Unknown \"splines\" value: \"%s\" - ignored\n", s);
+    agwarningf("Unknown \"splines\" value: \"%s\" - ignored\n", s);
     return defaultValue;
 }
 
-/* setEdgeType:
- * Sets graph's edge type based on the "splines" attribute.
+/** Sets graph's edge type based on the "splines" attribute.
  * If the attribute is not defined, use defaultValue.
  * If the attribute is "", use NONE.
  * If attribute value matches (case indepedent), use match.
@@ -1526,8 +1462,7 @@ void setEdgeType(graph_t *g, int defaultValue) {
     GD_flags(g) |= et;
 }
 
-/* get_gradient_points
- * Evaluates the extreme points of an ellipse or polygon
+/** Evaluates the extreme points of an ellipse or polygon
  * Determines the point at the center of the extreme points
  * If isRadial is true,sets the inner radius to half the distance to the min point;
  * else uses the angle parameter to identify two points on a line that defines the
@@ -1671,7 +1606,7 @@ static void fillMap (Agraph_t* g, Dt_t* map)
         Agraph_t *cl = GD_clust(g)[c];
         char *s = agnameof(cl);
         if (dtmatch(map, s)) {
-            agerr(AGWARN, "Two clusters named %s - the second will be ignored\n", s);
+            agwarningf("Two clusters named %s - the second will be ignored\n", s);
         } else {
             clust_t *ip = gv_alloc(sizeof(clust_t));
             ip->name = s;
@@ -1682,8 +1617,7 @@ static void fillMap (Agraph_t* g, Dt_t* map)
     }
 }
 
-/* mkClustMap:
- * Generates a dictionary mapping cluster names to corresponding cluster.
+/** Generates a dictionary mapping cluster names to corresponding cluster.
  * Used with cgraph as the latter does not support a flat namespace of clusters.
  * Assumes G has already built a cluster tree using GD_n_cluster and GD_clust.
  */
@@ -1702,11 +1636,17 @@ findCluster (Dt_t* map, char* name)
     clust_t* clp = dtmatch (map, name);
     if (clp)
 	return clp->clp;
-    else
-	return NULL;
+    return NULL;
 }
 
 /**
  * @dir lib/common
  * @brief common code for layout engines
+ * @ingroup engines
+ *
+ * @ref common_utils
+ *
+ * @defgroup common_utils utilities
+ * @brief low level utilities for layout engines and rendering
+ * @ingroup engines
  */
