@@ -1,9 +1,11 @@
+/// @file
+/// @ingroup common_utils
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -44,12 +46,8 @@ typedef struct {
   unsigned utf8 : 1;
 } xml_flags_t;
 
-UTILS_API nodequeue *new_queue(size_t);
-UTILS_API void free_queue(nodequeue *);
-UTILS_API void enqueue(nodequeue *, Agnode_t *);
-UTILS_API Agnode_t *dequeue(nodequeue *);
 UTILS_API pointf Bezier(pointf *, double, pointf *, pointf *);
-UTILS_API void attach_attrs_and_arrows(graph_t *, int *, int *);
+UTILS_API void attach_attrs_and_arrows(graph_t *, bool *, bool *);
 
 /** XML-escape a string
  *
@@ -98,7 +96,7 @@ UTILS_API bool findStopColor(char *colorlist, char *clrs[2], float *frac);
 UTILS_API int test_toggle(void);
 
 UTILS_API void common_init_node(node_t *n);
-UTILS_API int common_init_edge(edge_t *e);
+UTILS_API void common_init_edge(edge_t *e);
 
 UTILS_API void updateBB(graph_t *g, textlabel_t *lp);
 UTILS_API void compute_bb(Agraph_t *);
