@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -37,5 +37,5 @@ void setAspect(Agraph_t *g) {
     if (!p || ((r = sscanf (p, "%lf,%d", &rv, &passes)) <= 0)) {
 	return;
     }
-    agerr (AGWARN, "the aspect attribute has been disabled due to implementation flaws - attribute ignored.\n");
+    agwarningf("the aspect attribute has been disabled due to implementation flaws - attribute ignored.\n");
 }
