@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -21,6 +21,8 @@
 #include <cgraph/alloc.h>
 #include <cgraph/cgraph.h>
 #include <cgraph/exit.h>
+#include <cgraph/list.h>
+#include <cgraph/queue.h>
 #include <cgraph/streq.h>
 #include <dotgen/dot.h>
 #include <limits.h>
@@ -41,8 +43,8 @@ struct adjmatrix_t {
 
 	/* forward declarations */
 static bool medians(graph_t * g, int r0, int r1);
-static int nodeposcmpf(node_t ** n0, node_t ** n1);
-static int edgeidcmpf(edge_t ** e0, edge_t ** e1);
+static int nodeposcmpf(const void *, const void *);
+static int edgeidcmpf(const void *, const void *);
 static void flat_breakcycles(graph_t * g);
 static void flat_reorder(graph_t * g);
 static void flat_search(graph_t * g, node_t * v);
@@ -58,7 +60,7 @@ static void save_best(graph_t * g);
 static void restore_best(graph_t * g);
 static adjmatrix_t *new_matrix(size_t i, size_t j);
 static void free_matrix(adjmatrix_t * p);
-static int ordercmpf(int *i0, int *i1);
+static int ordercmpf(const void *, const void *);
 #ifdef DEBUG
 #if DEBUG > 1
 static int gd_minrank(Agraph_t *g) {return GD_minrank(g);}
@@ -75,7 +77,7 @@ void node_in_root_vlist(node_t * n);
 
 	/* mincross parameters */
 static int MinQuit;
-static double Convergence;
+static const double Convergence = .995;
 
 static graph_t *Root;
 static int GlobalMinRank, GlobalMaxRank;
@@ -275,7 +277,7 @@ fixLabelOrder (graph_t* g, rank_t* rk)
 	    int i, sz = agnnodes(sg);
 	    cnt = topsort (g, sg, arr);
 	    assert (cnt == sz);
-	    qsort(indices, cnt, sizeof(int), (qsort_cmpf)ordercmpf);
+	    qsort(indices, cnt, sizeof(int), ordercmpf);
 	    for (i = 0; i < sz; i++) {
 		ND_order(arr[i]) = indices[i];
 		rk->v[indices[i]] = arr[i];
@@ -454,7 +456,7 @@ static void do_ordering_node(graph_t *g, node_t *n, bool outflag) {
     /* write null terminator at end of list.
        requires +1 in TE_list alloccation */
     sortlist[ne] = 0;
-    qsort(sortlist, ne, sizeof(sortlist[0]), (qsort_cmpf) edgeidcmpf);
+    qsort(sortlist, ne, sizeof(sortlist[0]), edgeidcmpf);
     for (ne = 1; (f = sortlist[ne]); ne++) {
 	e = sortlist[ne - 1];
 	if (outflag) {
@@ -494,7 +496,7 @@ static void do_ordering_for_nodes(graph_t * g)
 	    else if (streq(ordering, "in"))
 		do_ordering_node(g, n, false);
 	    else if (ordering[0])
-		agerr(AGERR, "ordering '%s' not recognized for node '%s'.\n", ordering, agnameof(n));
+		agerrorf("ordering '%s' not recognized for node '%s'.\n", ordering, agnameof(n));
 	}
     }
 }
@@ -518,7 +520,7 @@ static void ordered_edges(graph_t * g)
 	else if (streq(ordering, "in"))
 	    do_ordering(g, false);
 	else if (ordering[0])
-	    agerr(AGERR, "ordering '%s' not recognized.\n", ordering);
+	    agerrorf("ordering '%s' not recognized.\n", ordering);
     }
     else
     {
@@ -759,7 +761,7 @@ static void restore_best(graph_t * g)
     for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
 	GD_rank(Root)[r].valid = false;
 	qsort(GD_rank(g)[r].v, GD_rank(g)[r].n, sizeof(GD_rank(g)[0].v[0]),
-	      (qsort_cmpf) nodeposcmpf);
+	      nodeposcmpf);
     }
 }
 
@@ -1164,7 +1166,7 @@ void allocate_ranks(graph_t * g)
     }
     GD_rank(g) = gv_calloc(GD_maxrank(g) + 2, sizeof(rank_t));
     for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
-	GD_rank(g)[r].an = GD_rank(g)[r].n = cn[r];
+	GD_rank(g)[r].an = GD_rank(g)[r].n = cn[r] + 1;
 	GD_rank(g)[r].av = GD_rank(g)[r].v = gv_calloc(cn[r] + 1, sizeof(node_t*));
     }
     free(cn);
@@ -1178,7 +1180,7 @@ void install_in_rank(graph_t * g, node_t * n)
     r = ND_rank(n);
     i = GD_rank(g)[r].n;
     if (GD_rank(g)[r].an <= 0) {
-	agerr(AGERR, "install_in_rank, line %d: %s %s rank %d i = %d an = 0\n",
+	agerrorf("install_in_rank, line %d: %s %s rank %d i = %d an = 0\n",
 	      __LINE__, agnameof(g), agnameof(n), r, i);
 	return;
     }
@@ -1198,18 +1200,18 @@ void install_in_rank(graph_t * g, node_t * n)
     }
 #endif
     if (ND_order(n) > GD_rank(Root)[r].an) {
-	agerr(AGERR, "install_in_rank, line %d: ND_order(%s) [%d] > GD_rank(Root)[%d].an [%d]\n",
+	agerrorf("install_in_rank, line %d: ND_order(%s) [%d] > GD_rank(Root)[%d].an [%d]\n",
 	      __LINE__, agnameof(n), ND_order(n), r, GD_rank(Root)[r].an);
 	return;
     }
     if (r < GD_minrank(g) || r > GD_maxrank(g)) {
-	agerr(AGERR, "install_in_rank, line %d: rank %d not in rank range [%d,%d]\n",
+	agerrorf("install_in_rank, line %d: rank %d not in rank range [%d,%d]\n",
 	      __LINE__, r, GD_minrank(g), GD_maxrank(g));
 	return;
     }
     if (GD_rank(g)[r].v + ND_order(n) >
 	GD_rank(g)[r].av + GD_rank(Root)[r].an) {
-	agerr(AGERR, "install_in_rank, line %d: GD_rank(g)[%d].v + ND_order(%s) [%d] > GD_rank(g)[%d].av + GD_rank(Root)[%d].an [%d]\n",
+	agerrorf("install_in_rank, line %d: GD_rank(g)[%d].v + ND_order(%s) [%d] > GD_rank(g)[%d].av + GD_rank(Root)[%d].an [%d]\n",
 	      __LINE__, r, agnameof(n),ND_order(n), r, r, GD_rank(Root)[r].an);
 	return;
     }
@@ -1224,9 +1226,7 @@ void build_ranks(graph_t * g, int pass)
     int i, j;
     node_t *n, *n0, *ns;
     edge_t **otheredges;
-    nodequeue *q;
-
-    q = new_queue(GD_n_nodes(g));
+    queue_t q = {0};
     for (n = GD_nlist(g); n; n = ND_next(n))
 	MARK(n) = false;
 
@@ -1261,19 +1261,18 @@ void build_ranks(graph_t * g, int pass)
 	    continue;
 	if (!MARK(n)) {
 	    MARK(n) = true;
-	    enqueue(q, n);
-	    while ((n0 = dequeue(q))) {
+	    queue_push(&q, n);
+	    while ((n0 = queue_pop(&q))) {
 		if (ND_ranktype(n0) != CLUSTER) {
 		    install_in_rank(g, n0);
-		    enqueue_neighbors(q, n0, pass);
+		    enqueue_neighbors(&q, n0, pass);
 		} else {
-		    install_cluster(g, n0, pass, q);
+		    install_cluster(g, n0, pass, &q);
 		}
 	    }
 	}
     }
-    if (dequeue(q))
-	agerr(AGERR, "surprise\n");
+    assert(queue_pop(&q) == NULL);
     for (i = GD_minrank(g); i <= GD_maxrank(g); i++) {
 	GD_rank(Root)[i].valid = false;
 	if (GD_flip(g) && GD_rank(g)[i].n > 0) {
@@ -1287,11 +1286,10 @@ void build_ranks(graph_t * g, int pass)
 
     if (g == dot_root(g) && ncross(g) > 0)
 	transpose(g, false);
-    free_queue(q);
+    queue_free(&q);
 }
 
-void enqueue_neighbors(nodequeue * q, node_t * n0, int pass)
-{
+void enqueue_neighbors(queue_t *q, node_t *n0, int pass) {
     edge_t *e;
 
     if (pass == 0) {
@@ -1299,7 +1297,7 @@ void enqueue_neighbors(nodequeue * q, node_t * n0, int pass)
 	    e = ND_out(n0).list[i];
 	    if (!MARK(aghead(e))) {
 		MARK(aghead(e)) = true;
-		enqueue(q, aghead(e));
+		queue_push(q, aghead(e));
 	    }
 	}
     } else {
@@ -1307,7 +1305,7 @@ void enqueue_neighbors(nodequeue * q, node_t * n0, int pass)
 	    e = ND_in(n0).list[i];
 	    if (!MARK(agtail(e))) {
 		MARK(agtail(e)) = true;
-		enqueue(q, agtail(e));
+		queue_push(q, agtail(e));
 	    }
 	}
     }
@@ -1323,32 +1321,32 @@ static bool constraining_flat_edge(Agraph_t *g, Agedge_t *e) {
   return true;
 }
 
+DEFINE_LIST(nodes, node_t *)
+
 /* construct nodes reachable from 'here' in post-order.
 * This is the same as doing a topological sort in reverse order.
 */
-static int postorder(graph_t * g, node_t * v, node_t ** list, int r)
-{
+static void postorder(graph_t *g, node_t *v, nodes_t *list, int r) {
     edge_t *e;
-    int i, cnt = 0;
+    int i;
 
     MARK(v) = true;
     if (ND_flat_out(v).size > 0) {
 	for (i = 0; (e = ND_flat_out(v).list[i]); i++) {
 	    if (!constraining_flat_edge(g, e)) continue;
 	    if (!MARK(aghead(e)))
-		cnt += postorder(g, aghead(e), list + cnt, r);
+		postorder(g, aghead(e), list, r);
 	}
     }
     assert(ND_rank(v) == r);
-    list[cnt++] = v;
-    return cnt;
+    nodes_append(list, v);
 }
 
 static void flat_reorder(graph_t * g)
 {
-    int i, r, pos, n_search, local_in_cnt, local_out_cnt, base_order;
-    node_t *v, **left, **right, *t;
-    node_t **temprank = NULL;
+    int i, r, local_in_cnt, local_out_cnt, base_order;
+    node_t *v;
+    nodes_t temprank = {0};
     edge_t *flat_e, *e;
 
     if (!GD_has_flat_edges(g))
@@ -1358,8 +1356,7 @@ static void flat_reorder(graph_t * g)
 	base_order = ND_order(GD_rank(g)[r].v[0]);
 	for (i = 0; i < GD_rank(g)[r].n; i++)
 	    MARK(GD_rank(g)[r].v[i]) = false;
-	temprank = ALLOC(i + 1, temprank, node_t *);
-	pos = 0;
+	nodes_clear(&temprank);
 
 	/* construct reverse topological sort order in temprank */
 	for (i = 0; i < GD_rank(g)[r].n; i++) {
@@ -1376,30 +1373,20 @@ static void flat_reorder(graph_t * g)
 		if (constraining_flat_edge(g, flat_e)) local_out_cnt++;
 	    }
 	    if ((local_in_cnt == 0) && (local_out_cnt == 0))
-		temprank[pos++] = v;
+		nodes_append(&temprank, v);
 	    else {
 		if (!MARK(v) && local_in_cnt == 0) {
-		    left = temprank + pos;
-		    n_search = postorder(g, v, left, r);
-		    pos += n_search;
+		    postorder(g, v, &temprank, r);
 		}
 	    }
 	}
 
-	if (pos) {
+	if (nodes_size(&temprank) > 0) {
 	    if (!GD_flip(g)) {
-		left = temprank;
-		right = temprank + pos - 1;
-		while (left < right) {
-		    t = *left;
-		    *left = *right;
-		    *right = t;
-		    left++;
-		    right--;
-		}
+		nodes_reverse(&temprank);
 	    }
 	    for (i = 0; i < GD_rank(g)[r].n; i++) {
-		v = GD_rank(g)[r].v[i] = temprank[i];
+		v = GD_rank(g)[r].v[i] = nodes_get(&temprank, (size_t)i);
 		ND_order(v) = i + base_order;
 	    }
 
@@ -1423,7 +1410,7 @@ static void flat_reorder(graph_t * g)
 	/* else do no harm! */
 	GD_rank(Root)[r].valid = false;
     }
-    free(temprank);
+    nodes_free(&temprank);
 }
 
 static void reorder(graph_t * g, int r, bool reverse, bool hasfixed)
@@ -1586,8 +1573,9 @@ int ncross(graph_t * g)
     return count;
 }
 
-static int ordercmpf(int *i0, int *i1)
-{
+static int ordercmpf(const void *x, const void *y) {
+  const int *i0 = x;
+  const int *i1 = y;
   if (*i0 < *i1) {
     return -1;
   }
@@ -1671,7 +1659,7 @@ static bool medians(graph_t * g, int r0, int r1)
 	    ND_mval(n) = (list[0] + list[1]) / 2;
 	    break;
 	default:
-	    qsort(list, j, sizeof(int), (qsort_cmpf) ordercmpf);
+	    qsort(list, j, sizeof(int), ordercmpf);
 	    if (j % 2)
 		ND_mval(n) = list[j / 2];
 	    else {
@@ -1697,8 +1685,19 @@ static bool medians(graph_t * g, int r0, int r1)
     return hasfixed;
 }
 
-static int nodeposcmpf(node_t ** n0, node_t ** n1)
-{
+static int nodeposcmpf(const void *x, const void *y) {
+// Suppress Clang/GCC -Wcast-qual warning. Casting away const here is acceptable
+// as the later usage is const. We need the cast because the macros use
+// non-const pointers for genericity.
+#ifdef __GNUC__
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-qual"
+#endif
+  node_t **n0 = (node_t **)x;
+  node_t **n1 = (node_t **)y;
+#ifdef __GNUC__
+#pragma GCC diagnostic pop
+#endif
   if (ND_order(*n0) < ND_order(*n1)) {
     return -1;
   }
@@ -1708,8 +1707,19 @@ static int nodeposcmpf(node_t ** n0, node_t ** n1)
   return 0;
 }
 
-static int edgeidcmpf(edge_t ** e0, edge_t ** e1)
-{
+static int edgeidcmpf(const void *x, const void *y) {
+// Suppress Clang/GCC -Wcast-qual warning. Casting away const here is acceptable
+// as the later usage is const. We need the cast because the macros use
+// non-const pointers for genericity.
+#ifdef __GNUC__
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-qual"
+#endif
+  edge_t **e0 = (edge_t **)x;
+  edge_t **e1 = (edge_t **)y;
+#ifdef __GNUC__
+#pragma GCC diagnostic pop
+#endif
   if (AGSEQ(*e0) < AGSEQ(*e1)) {
     return -1;
   }
@@ -1753,7 +1763,7 @@ void virtual_weight(edge_t * e)
     /* check whether the upcoming computation will overflow */
     assert(t >= 0);
     if (INT_MAX / t < ED_weight(e)) {
-	agerr(AGERR, "overflow when calculating virtual weight of edge\n");
+	agerrorf("overflow when calculating virtual weight of edge\n");
 	graphviz_exit(EXIT_FAILURE);
     }
 
@@ -1811,7 +1821,6 @@ static void mincross_options(graph_t * g)
     /* set default values */
     MinQuit = 8;
     MaxIter = 24;
-    Convergence = .995;
 
     p = agget(g, "mclimit");
     if (p && (f = atof(p)) > 0.0) {
