@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -17,8 +17,9 @@
  * created and correctly separated.
  */
 
+#include <common/geomprocs.h>
 #include <cgraph/alloc.h>
-#include <cgraph/clamp.h>
+#include <cgraph/gv_math.h>
 #include <dotgen/dot.h>
 #include <dotgen/aspect.h>
 #include <math.h>
@@ -60,7 +61,7 @@ dumpNS (graph_t * g)
 static double
 largeMinlen (double l)
 {
-  agerr(AGERR,
+  agerrorf(
         "Edge length %f larger than maximum %d allowed.\nCheck for overwide "
         "node(s).\n",
         l, INT_MAX);
@@ -924,16 +925,13 @@ static void set_aspect(graph_t *g) {
     double xf = 0.0, yf = 0.0, actual, desired;
     node_t *n;
     bool filled;
-    point sz;
 
     rec_bb(g, g);
     if (GD_maxrank(g) > 0 && GD_drawing(g)->ratio_kind) {
-	sz.x = GD_bb(g).UR.x - GD_bb(g).LL.x;
-	sz.y = GD_bb(g).UR.y - GD_bb(g).LL.y;	/* normalize */
+	pointf sz = {.x = GD_bb(g).UR.x - GD_bb(g).LL.x,
+	             .y = GD_bb(g).UR.y - GD_bb(g).LL.y}; // normalize
 	if (GD_flip(g)) {
-	    int t = sz.x;
-	    sz.x = sz.y;
-	    sz.y = t;
+	    sz = exch_xyf(sz);
 	}
 	bool scale_it = true;
 	if (GD_drawing(g)->ratio_kind == R_AUTO)
@@ -945,14 +943,14 @@ static void set_aspect(graph_t *g) {
 	    if (GD_drawing(g)->size.x <= 0)
 		scale_it = false;
 	    else {
-		xf = (double) GD_drawing(g)->size.x / (double) sz.x;
-		yf = (double) GD_drawing(g)->size.y / (double) sz.y;
+		xf = GD_drawing(g)->size.x / sz.x;
+		yf = GD_drawing(g)->size.y / sz.y;
 		if (xf < 1.0 || yf < 1.0) {
 		    if (xf < yf) {
-			yf = yf / xf;
+			yf /= xf;
 			xf = 1.0;
 		    } else {
-			xf = xf / yf;
+			xf /= yf;
 			yf = 1.0;
 		    }
 		}
@@ -961,19 +959,17 @@ static void set_aspect(graph_t *g) {
 	    if (GD_drawing(g)->size.x <= 0)
 		scale_it = false;
 	    else {
-		xf = (double) GD_drawing(g)->size.x /
-		    (double) GD_bb(g).UR.x;
-		yf = (double) GD_drawing(g)->size.y /
-		    (double) GD_bb(g).UR.y;
-		if ((xf > 1.0) && (yf > 1.0)) {
-		    double scale = MIN(xf, yf);
+		xf = GD_drawing(g)->size.x / GD_bb(g).UR.x;
+		yf = GD_drawing(g)->size.y / GD_bb(g).UR.y;
+		if (xf > 1.0 && yf > 1.0) {
+		    const double scale = fmin(xf, yf);
 		    xf = yf = scale;
 		} else
 		    scale_it = false;
 	    }
 	} else if (GD_drawing(g)->ratio_kind == R_VALUE) {
 	    desired = GD_drawing(g)->ratio;
-	    actual = ((double) sz.y) / ((double) sz.x);
+	    actual = sz.y / sz.x;
 	    if (actual < desired) {
 		yf = desired / actual;
 		xf = 1.0;
@@ -990,8 +986,8 @@ static void set_aspect(graph_t *g) {
 		yf = t;
 	    }
 	    for (n = GD_nlist(g); n; n = ND_next(n)) {
-		ND_coord(n).x = ROUND(ND_coord(n).x * xf);
-		ND_coord(n).y = ROUND(ND_coord(n).y * yf);
+		ND_coord(n).x = round(ND_coord(n).x * xf);
+		ND_coord(n).y = round(ND_coord(n).y * yf);
 	    }
 	    scale_bb(g, g, xf, yf);
 	}
@@ -1115,7 +1111,7 @@ static void contain_nodes(graph_t * g)
 	    continue;
 	v = GD_rank(g)[r].v[0];
 	if (v == NULL) {
-	    agerr(AGERR, "contain_nodes clust %s rank %d missing node\n",
+	    agerrorf("contain_nodes clust %s rank %d missing node\n",
 		  agnameof(g), r);
 	    continue;
 	}
