@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -43,8 +43,7 @@ static Switch_t		swstate;
 
 Exstate_t		expr;
 
-static int T(int t) 
-{
+static int T(long t) {
 	if (expr.program->disc->types)
 	    return expr.program->disc->types[t & TMASK];
 	else
@@ -55,9 +54,8 @@ static int T(int t)
  * allocate and initialize a new expression node in the current program
  */
 
-Exnode_t*
-exnewnode(Expr_t* p, int op, int binary, int type, Exnode_t* left, Exnode_t* right)
-{
+Exnode_t *exnewnode(Expr_t *p, int op, int binary, long type, Exnode_t *left,
+                    Exnode_t *right) {
 	Exnode_t*	x;
 
 	x = ALLOCATE(p, Exnode_t);
@@ -65,8 +63,7 @@ exnewnode(Expr_t* p, int op, int binary, int type, Exnode_t* left, Exnode_t* rig
 	x->op = op;
 	x->type = type;
 	x->binary = binary;
-	x->local.number = 0;
-	x->local.pointer = 0;
+	x->local = NULL;
 	x->data.operand.left = left;
 	x->data.operand.right = right;
 	return x;
@@ -99,25 +96,25 @@ exfreenode(Expr_t* p, Exnode_t* x)
 	case DYNAMIC:
 		if (x->data.variable.index)
 			exfreenode(p, x->data.variable.index);
-		if (x->data.variable.symbol->local.pointer)
+		if (x->data.variable.symbol->local)
 		{
-			dtclose((Dt_t*)x->data.variable.symbol->local.pointer);
-			x->data.variable.symbol->local.pointer = 0;
+			dtclose(x->data.variable.symbol->local);
+			x->data.variable.symbol->local = NULL;
 		}
 		break;
 	case '#':
-		if (x->data.variable.symbol->local.pointer) {
-			dtclose((Dt_t *) x->data.variable.symbol->local.pointer);
-			x->data.variable.symbol->local.pointer = 0;
+		if (x->data.variable.symbol->local) {
+			dtclose(x->data.variable.symbol->local);
+			x->data.variable.symbol->local = NULL;
 		}
 		break;
 //	case IN_OP:
 	case UNSET:
 		if (x->data.variable.index)
 			exfreenode(p, x->data.variable.index);
-		if (x->data.variable.symbol->local.pointer) {
-			dtclose((Dt_t *) x->data.variable.symbol->local.pointer);
-			x->data.variable.symbol->local.pointer = 0;
+		if (x->data.variable.symbol->local) {
+			dtclose(x->data.variable.symbol->local);
+			x->data.variable.symbol->local = NULL;
 		}
 		break;
 	case ITERATE:
@@ -148,9 +145,9 @@ exfreenode(Expr_t* p, Exnode_t* x)
 		if (x->data.split.seps)
 			exfreenode(p, x->data.split.seps);
 		exfreenode(p, x->data.split.string);
-		if (x->data.split.array->local.pointer) {
-			dtclose((Dt_t *) x->data.split.array->local.pointer);
-			x->data.split.array->local.pointer = 0;
+		if (x->data.split.array->local) {
+			dtclose(x->data.split.array->local);
+			x->data.split.array->local = NULL;
 		}
 		break;
 	case PRINT:
@@ -208,7 +205,7 @@ static Exnode_t *extract(Expr_t * p, Exnode_t ** argp, int type) {
 static Exnode_t *exnewsplit(Expr_t * p, int op, Exid_t* dyn, Exnode_t * s, Exnode_t* seps) {
 	Exnode_t *ss = 0;
 
-	if (dyn->local.pointer == 0)
+	if (dyn->local == NULL)
               	exerror("cannot use non-array %s in %s", dyn->name, exopname(op));
 	if ((dyn->index_type > 0) && (dyn->index_type != INTEGER))
             exerror("in %s, array %s must have integer index type, not %s", 
@@ -295,7 +292,7 @@ static Exnode_t *exnewsubstr(Expr_t * p, Exnode_t * args) {
  * Assume x->type != STRING
  */
 static Exnode_t *exstringOf(Expr_t * p, Exnode_t * x) {
-	int type = x->type;
+	const long type = x->type;
 	int cvt = 0;
 
 	if (!type) {
@@ -333,10 +330,10 @@ static Exnode_t *exstringOf(Expr_t * p, Exnode_t * x) {
 		break;
 	    case INTEGER:
 		x->data.constant.value.string =
-		  exprintf(p->vm, "%lld", (long long)x->data.constant.value.integer);
+		  exprintf(p->vm, "%lld", x->data.constant.value.integer);
 		break;
 	    default:
-		exerror("internal error: %d: unknown type", type);
+		exerror("internal error: %ld: unknown type", type);
 		break;
 	    }
 	x->type = STRING;
@@ -371,7 +368,6 @@ static Exnode_t *exprint(Expr_t * p, Exid_t * ex, Exnode_t * args) {
 static Exnode_t *makeVar(Expr_t * prog, Exid_t * s, Exnode_t * idx,
 			     Exnode_t * dyna, Exref_t * refs) {
 	Exnode_t *nn;
-	int kind;
 	Exid_t *sym;
 
 	/* parse components */
@@ -386,10 +382,7 @@ static Exnode_t *makeVar(Expr_t * prog, Exid_t * s, Exnode_t * idx,
 	} else
 	    sym = s;
 
-	if (sym->type)
-	    kind = sym->type;
-	else
-	    kind = STRING;
+	const long kind = sym->type ? sym->type : STRING;
 
 	nn = exnewnode(prog, ID, 0, kind, NULL, NULL);
 	nn->data.variable.symbol = sym;
@@ -429,7 +422,7 @@ static int	typecast[6][6] =
 
 #define EXTERNAL(t)	((t)>=F2X)
 
-char *extypename(Expr_t * p, int type) {
+char *extypename(Expr_t *p, long type) {
 	if (BUILTIN(type))
 	    return TYPENAME(type);
 	return p->disc->typename(type);
@@ -444,9 +437,7 @@ Exnode_t *exnoncast(Exnode_t * x) {
 	return x;
 }
 
-Exnode_t*
-excast(Expr_t* p, Exnode_t* x, int type, Exnode_t* xref, int arg)
-{
+Exnode_t *excast(Expr_t *p, Exnode_t *x, long type, Exnode_t *xref, int arg) {
 	int	t2t;
 	char*		s;
 	char*		e;
@@ -513,7 +504,7 @@ excast(Expr_t* p, Exnode_t* x, int type, Exnode_t* xref, int arg)
 			break;
 		case I2S:
 			x->data.constant.value.string =
-			  exprintf(p->vm, "%lld", (long long)x->data.constant.value.integer);
+			  exprintf(p->vm, "%lld", x->data.constant.value.integer);
 			break;
 		case S2F:
 			s =  x->data.constant.value.string;
@@ -737,12 +728,12 @@ preprint(Exnode_t* args)
 			{
 			case FLOATING:
 				if (x->arg->type != FLOATING)
-					x->arg = exnewnode(expr.program, x->arg->type == STRING ? S2F : INTEGRAL(x->arg->type) ? I2F : X2F, 0, FLOATING, x->arg, x->arg->op == ID ? x->arg : (Exnode_t*)0);
+					x->arg = exnewnode(expr.program, x->arg->type == STRING ? S2F : INTEGRAL(x->arg->type) ? I2F : X2F, 0, FLOATING, x->arg, x->arg->op == ID ? x->arg : NULL);
 				break;
 			case INTEGER:
 			case UNSIGNED:
 				if (!INTEGRAL(x->arg->type))
-					x->arg = exnewnode(expr.program, x->arg->type == STRING ? S2I : x->arg->type == FLOATING ? F2I : X2I, 0, INTEGER, x->arg, x->arg->op == ID ? x->arg : (Exnode_t*)0);
+					x->arg = exnewnode(expr.program, x->arg->type == STRING ? S2I : x->arg->type == FLOATING ? F2I : X2I, 0, INTEGER, x->arg, x->arg->op == ID ? x->arg : NULL);
 				x->arg->type = t;
 				break;
 			case STRING:
@@ -757,7 +748,7 @@ preprint(Exnode_t* args)
 					else if (!expr.program->disc->convertf || (x->arg->op != ID && x->arg->op != DYNAMIC && x->arg->op != F2X && x->arg->op != I2X && x->arg->op != S2X))
 						exerror("string format argument expected");
 					else
-						x->arg = exnewnode(expr.program, x->arg->type == FLOATING ? F2S : INTEGRAL(x->arg->type) ? I2S : X2S, 0, STRING, x->arg, x->arg->op == ID ? x->arg : (Exnode_t*)0);
+						x->arg = exnewnode(expr.program, x->arg->type == FLOATING ? F2S : INTEGRAL(x->arg->type) ? I2S : X2S, 0, STRING, x->arg, x->arg->op == ID ? x->arg : NULL);
 				}
 				break;
 			}
