@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -73,14 +73,12 @@ typedef struct {
  * node's connected component.
  * Also, entire layout is translated to origin.
  */
-static void
-finalCC(graph_t * g, int c_cnt, graph_t ** cc, point * pts, graph_t * rg,
-	layout_info* infop)
-{
+static void finalCC(graph_t *g, size_t c_cnt, graph_t **cc, point *pts,
+                    graph_t *rg, layout_info* infop) {
     attrsym_t * G_width = infop->G_width;
     attrsym_t * G_height = infop->G_height;
     graph_t *cg;
-    box b, bb;
+    boxf bb;
     boxf bbf;
     point pt;
     int margin;
@@ -92,7 +90,7 @@ finalCC(graph_t * g, int c_cnt, graph_t ** cc, point * pts, graph_t * rg,
     /* compute graph bounding box in points */
     if (c_cnt) {
 	cg = *cp++;
-	BF2B(GD_bb(cg), bb);
+	bb = GD_bb(cg);
 	if (c_cnt > 1) {
 	    pt = *pp++;
 	    bb.LL.x += pt.x;
@@ -100,16 +98,16 @@ finalCC(graph_t * g, int c_cnt, graph_t ** cc, point * pts, graph_t * rg,
 	    bb.UR.x += pt.x;
 	    bb.UR.y += pt.y;
 	    while ((cg = *cp++)) {
-		BF2B(GD_bb(cg), b);
+		boxf b = GD_bb(cg);
 		pt = *pp++;
 		b.LL.x += pt.x;
 		b.LL.y += pt.y;
 		b.UR.x += pt.x;
 		b.UR.y += pt.y;
-		bb.LL.x = MIN(bb.LL.x, b.LL.x);
-		bb.LL.y = MIN(bb.LL.y, b.LL.y);
-		bb.UR.x = MAX(bb.UR.x, b.UR.x);
-		bb.UR.y = MAX(bb.UR.y, b.UR.y);
+		bb.LL.x = fmin(bb.LL.x, b.LL.x);
+		bb.LL.y = fmin(bb.LL.y, b.LL.y);
+		bb.UR.x = fmax(bb.UR.x, b.UR.x);
+		bb.UR.y = fmax(bb.UR.y, b.UR.y);
 	    }
 	}
     } else {			/* empty graph */
@@ -122,11 +120,10 @@ finalCC(graph_t * g, int c_cnt, graph_t ** cc, point * pts, graph_t * rg,
 
     if (GD_label(rg)) {
 	point p;
-	int d;
 
 	isEmpty = 0;
 	PF2P(GD_label(rg)->dimen, p);
-	d = p.x - (bb.UR.x - bb.LL.x);
+	double d = p.x - (bb.UR.x - bb.LL.x);
 	if (d > 0) {		/* height of label added below */
 	    d /= 2;
 	    bb.LL.x -= d;
@@ -341,7 +338,7 @@ static void chkPos(graph_t* g, node_t* n, layout_info* infop, boxf* bbp)
 		ND_pinned(n) = P_SET;
 	    *bbp = bb;
 	} else
-	    agerr(AGWARN, "graph %s, coord %s, expected four doubles\n",
+	    agwarningf("graph %s, coord %s, expected four doubles\n",
 		  agnameof(g), p);
     }
 }
@@ -440,7 +437,7 @@ static graph_t *deriveGraph(graph_t * g, layout_info * infop)
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	if (!DNODE(n)) {
 	    if (PARENT(n) && PARENT(n) != GPARENT(g)) {
-		agerr (AGERR, "node \"%s\" is contained in two non-comparable clusters \"%s\" and \"%s\"\n", agnameof(n), agnameof(g), agnameof(PARENT(n)));
+		agerrorf("node \"%s\" is contained in two non-comparable clusters \"%s\" and \"%s\"\n", agnameof(n), agnameof(g), agnameof(PARENT(n)));
 		return NULL;
 	    }
 	    PARENT(n) = g;
@@ -821,7 +818,6 @@ static int layout(graph_t * g, layout_info * infop)
     graph_t *sg;
     graph_t **cc;
     graph_t **pg;
-    int c_cnt;
     int pinned;
     xparams xpms;
 
@@ -842,6 +838,7 @@ static int layout(graph_t * g, layout_info * infop)
     if (dg == NULL) {
 	return -1;
     }
+    size_t c_cnt;
     cc = pg = findCComp(dg, &c_cnt, &pinned);
 
     while ((cg = *pg++)) {
@@ -1069,7 +1066,7 @@ fdpSplines (graph_t * g)
 	}
 	if (trySplines || et != EDGETYPE_COMPOUND) {
 	    if (HAS_CLUST_EDGE(g)) {
-		agerr(AGWARN,
+		agwarningf(
 		      "splines and cluster edges not supported - using line segments\n");
 		et = EDGETYPE_LINE;
 	    } else {
