@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -18,7 +18,6 @@
 
 #include <assert.h>
 #include <string.h>
-#include <common/memory.h>
 #include <common/const.h>
 #include <common/macros.h>
 #include <common/colorprocs.h>
@@ -44,7 +43,7 @@ int gvrender_select(GVJ_t * job, const char *str)
     gvplugin_available_t *plugin;
     gvplugin_installed_t *typeptr;
 
-    gvplugin_load(gvc, API_device, str);
+    gvplugin_load(gvc, API_device, str, NULL);
 
     /* When job is created, it is zeroed out.
      * Some flags, such as OUTPUT_NOT_REQUIRED, may already be set,
@@ -177,15 +176,14 @@ pointf *gvrender_ptf_A(GVJ_t *job, pointf *af, pointf *AF, size_t n) {
 
 static int gvrender_comparestr(const void *s1, const void *s2)
 {
-    return strcmp(*(char *const *) s1, *(char *const *) s2);
+  return strcmp(s1, *(char *const *) s2);
 }
 
 /* gvrender_resolve_color:
  * N.B. strcmp cannot be used in bsearch, as it will pass a pointer
  * to an element in the array features->knowncolors (i.e., a char**)
  * as an argument of the compare function, while the arguments to 
- * strcmp are both char*. Given this, the first argument to
- * bsearch must also be char**, so we use &tok.
+ * strcmp are both char*.
  */
 static void gvrender_resolve_color(gvrender_features_t * features,
 				   char *name, gvcolor_t * color)
@@ -198,8 +196,7 @@ static void gvrender_resolve_color(gvrender_features_t * features,
     tok = canontoken(name);
     if (!features->knowncolors
 	||
-	(bsearch
-	 (&tok, features->knowncolors, features->sz_knowncolors,
+	(bsearch(tok, features->knowncolors, features->sz_knowncolors,
 	  sizeof(char *), gvrender_comparestr)) == NULL) {
 	/* if tok was not found in known_colors */
 	rc = colorxlate(name, color, features->color_type);
@@ -208,13 +205,14 @@ static void gvrender_resolve_color(gvrender_features_t * features,
 		agxbuf missedcolor = {0};
 		agxbprint(&missedcolor, "color %s", name);
 		if (emit_once(agxbuse(&missedcolor)))
-		    agerr(AGWARN, "%s is not a known color.\n", name);
+		    agwarningf("%s is not a known color.\n", name);
 		agxbfree(&missedcolor);
 	    } else {
-		agerr(AGERR, "error in colxlate()\n");
+		agerrorf("error in colxlate()\n");
 	    }
 	}
     }
+    free(tok);
 }
 
 void gvrender_begin_graph(GVJ_t *job) {
@@ -514,7 +512,7 @@ void gvrender_set_style(GVJ_t * job, char **s)
 		    obj->fill = FILL_NONE;
 		else if (streq(line, "tapered"));
 		else {
-		    agerr(AGWARN,
+		    agwarningf(
 			  "gvrender_set_style: unsupported style %s - ignoring\n",
 			  line);
 		}
@@ -527,13 +525,10 @@ void gvrender_ellipse(GVJ_t *job, pointf *pf, int filled) {
 
     if (gvre) {
 	if (gvre->ellipse && job->obj->pen != PEN_NONE) {
-	    pointf af[2];
-
-	    /* center */
-	    af[0].x = (pf[0].x + pf[1].x) / 2.;
-	    af[0].y = (pf[0].y + pf[1].y) / 2.;
-	    /* corner */
-	    af[1] = pf[1];
+	    pointf af[] = {
+		mid_pointf(pf[0], pf[1]), // center
+		pf[1] // corner
+	    };
 
 	    if (!(job->flags & GVRENDER_DOES_TRANSFORM))
 		gvrender_ptf_A(job, af, af, 2);
