@@ -3,13 +3,15 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #include "config.h"
+#include <assert.h>
 #include <limits.h>
+#include <math.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <string.h>
@@ -62,7 +64,6 @@ typedef struct {
 #define SVG_MAGIC  "<svg"
 #define RIFF_MAGIC "RIFF"
 #define WEBP_MAGIC "WEBP"
-//#define TIFF_MAGIC "II"
 #define ICO_MAGIC  "\x00\x00\x01\x00"
 
 static knowntype_t knowntypes[] = {
@@ -76,7 +77,6 @@ static knowntype_t knowntypes[] = {
     { XML_MAGIC,  sizeof(XML_MAGIC)-1,   FT_XML,  "xml",  },
     { RIFF_MAGIC, sizeof(RIFF_MAGIC)-1,  FT_RIFF, "riff", },
     { ICO_MAGIC,  sizeof(ICO_MAGIC)-1,   FT_ICO,  "ico",  },
-//    { TIFF_MAGIC, sizeof(TIFF_MAGIC)-1,  FT_TIFF, "tiff", },
 };
 
 static imagetype_t imagetype(usershape_t *us) {
@@ -170,19 +170,19 @@ static bool get_int_msb_first(FILE *f, size_t sz, int *val) {
     return true;
 }
 
-static int svg_units_convert(double n, char *u) {
+static double svg_units_convert(double n, char *u) {
     if (strcmp(u, "in") == 0)
-	return ROUND(n * POINTS_PER_INCH);
+	return round(n * POINTS_PER_INCH);
     if (strcmp(u, "px") == 0)
-        return ROUND(n * POINTS_PER_INCH / 96);
+        return round(n * POINTS_PER_INCH / 96);
     if (strcmp(u, "pc") == 0)
-        return ROUND(n * POINTS_PER_INCH / 6); 
+        return round(n * POINTS_PER_INCH / 6); 
     if (strcmp(u, "pt") == 0 || strcmp(u, "\"") == 0)   /* ugly!!  - if there are no inits then the %2s get the trailing '"' */
-        return ROUND(n);
+        return round(n);
     if (strcmp(u, "cm") == 0)
-        return ROUND(n * POINTS_PER_CM);
+        return round(n * POINTS_PER_CM);
     if (strcmp(u, "mm") == 0)
-        return ROUND(n * POINTS_PER_MM);
+        return round(n * POINTS_PER_MM);
     return 0;
 }
 
@@ -227,7 +227,7 @@ static int find_attribute(const char *s, match_t *result) {
 
 static void svg_size (usershape_t *us)
 {
-    int w = 0, h = 0;
+    double w = 0, h = 0;
     double n, x0, y0, x1, y1;
     char u[10];
     agxbuf line = {0};
@@ -296,8 +296,10 @@ static void svg_size (usershape_t *us)
 	}
     }
     us->dpi = 0;
-    us->w = w;
-    us->h = h;
+    assert(w >= 0 && w <= INT_MAX);
+    us->w = (int)w;
+    assert(h >= 0 && h <= INT_MAX);
+    us->h = (int)h;
     agxbfree(&line);
 }
 
@@ -633,12 +635,12 @@ bool gvusershape_file_access(usershape_t *us)
 	fseek(us->f, 0, SEEK_SET);
     else {
         if (! (fn = safefile(us->name))) {
-	    agerr(AGWARN, "Filename \"%s\" is unsafe\n", us->name);
+	    agwarningf("Filename \"%s\" is unsafe\n", us->name);
 	    return false;
 	}
 	us->f = fopen(fn, "rb");
 	if (us->f == NULL) {
-	    agerr(AGWARN, "%s while opening %s\n", strerror(errno), fn);
+	    agwarningf("%s while opening %s\n", strerror(errno), fn);
 	    return false;
 	}
 	if (usershape_files_open_cnt >= MAX_USERSHAPE_FILES_OPEN)
@@ -689,7 +691,7 @@ static usershape_t *gvusershape_open (const char *name)
         switch(imagetype(us)) {
 	    case FT_NULL:
 		if (!(us->data = find_user_shape(us->name))) {
-		    agerr(AGWARN, "\"%s\" was not found as a file or as a shape library member\n", us->name);
+		    agwarningf("\"%s\" was not found as a file or as a shape library member\n", us->name);
 		    freeUsershape (us);
 		    return NULL;
 		}
