@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -16,10 +16,10 @@
 #include <ast/error.h>
 #include <cgraph/agxbuf.h>
 #include <cgraph/alloc.h>
+#include <cgraph/gv_ctype.h>
 #include <cgraph/strcasecmp.h>
 #include <cgraph/unreachable.h>
 #include <cgraph/unused.h>
-#include <ctype.h>
 #include <gvpr/actions.h>
 #include <gvpr/compile.h>
 #include <limits.h>
@@ -105,7 +105,7 @@ long rindexOf(char *s1, char *s2) {
 int match(char *str, char *pat) {
   int sub[2];
 
-  if (strgrpmatch(str, pat, sub, 1, STR_MAXIMAL)) {
+  if (strgrpmatch(str, pat, sub, 1, 0)) {
     return (sub[0]);
   } else
     return -1;
@@ -598,25 +598,21 @@ Agraph_t *readFile(char *f) {
   return gp;
 }
 
-int fwriteFile(Expr_t *ex, Agraph_t *g, int fd) {
+int fwriteFile(Expr_t *ex, Agraph_t *g, long long fd) {
   FILE *sp;
 
-  if (fd < 0 ||
-      (elementsof(ex->file) <= INT_MAX && fd >= (int)elementsof(ex->file)) ||
-      !((sp = ex->file[fd]))) {
-    exerror("fwriteG: %d: invalid descriptor", fd);
+  if (fd < 0 || fd >= (long long)elementsof(ex->file) || !(sp = ex->file[fd])) {
+    exerror("fwriteG: %lld: invalid descriptor", fd);
     return 0;
   }
   return sfioWrite(g, sp);
 }
 
-Agraph_t *freadFile(Expr_t *ex, int fd) {
+Agraph_t *freadFile(Expr_t *ex, long long fd) {
   FILE *sp;
 
-  if (fd < 0 ||
-      (elementsof(ex->file) <= INT_MAX && fd >= (int)elementsof(ex->file)) ||
-      !((sp = ex->file[fd]))) {
-    exerror("freadG: %d: invalid descriptor", fd);
+  if (fd < 0 || fd >= (long long)elementsof(ex->file) || !(sp = ex->file[fd])) {
+    exerror("freadG: %lld: invalid descriptor", fd);
     return 0;
   }
   return readG(sp);
@@ -640,20 +636,19 @@ int openFile(Expr_t *ex, const char *fname, const char *mode) {
     return -1;
 }
 
-int closeFile(Expr_t *ex, int fd) {
+int closeFile(Expr_t *ex, long long fd) {
   int rv;
 
   if (0 <= fd && fd <= 2) {
-    exerror("closeF: cannot close standard stream %d", fd);
+    exerror("closeF: cannot close standard stream %lld", fd);
     return -1;
   }
-  if (fd < 0 ||
-      (elementsof(ex->file) <= INT_MAX && fd >= (int)elementsof(ex->file))) {
-    exerror("closeG: %d: invalid descriptor", fd);
+  if (fd < 0 || fd >= (long long)elementsof(ex->file)) {
+    exerror("closeG: %lld: invalid descriptor", fd);
     return -1;
   }
   if (!ex->file[fd]) {
-    exerror("closeF: stream %d not open", fd);
+    exerror("closeF: stream %lld not open", fd);
     return -1;
   }
   rv = fclose(ex->file[fd]);
@@ -666,13 +661,13 @@ int closeFile(Expr_t *ex, int fd) {
  * Read single line from stream.
  * Return "" on EOF.
  */
-char *readLine(Expr_t *ex, int fd) {
+char *readLine(Expr_t *ex, long long fd) {
   FILE *sp;
   int c;
   char *line;
 
-  if (fd < 0 || fd >= elementsof(ex->file) || !((sp = ex->file[fd]))) {
-    exerror("readL: %d: invalid descriptor", fd);
+  if (fd < 0 || fd >= (long long)elementsof(ex->file) || !(sp = ex->file[fd])) {
+    exerror("readL: %lld: invalid descriptor", fd);
     return "";
   }
 
@@ -729,7 +724,7 @@ char *toLower(Expr_t *pgm, char *src) {
   }
 
   for (size_t i = 0; i < len; ++i) {
-    dst[i] = (char)tolower((int)src[i]);
+    dst[i] = gv_tolower(src[i]);
   }
 
   dst[len] = '\0';
@@ -748,7 +743,7 @@ char *toUpper(Expr_t *pgm, char *src) {
   }
 
   for (size_t i = 0; i < len; ++i) {
-    dst[i] = (char)toupper((int)src[i]);
+    dst[i] = gv_toupper(src[i]);
   }
 
   dst[len] = '\0';
