@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -49,12 +49,12 @@ static int isedge(Agobj_t *obj) {
 
 #define BITS_PER_BYTE 8
 
-static void *int2ptr(Sflong_t i) {
+static void *int2ptr(long long i) {
   return (void*)(intptr_t)i;
 }
 
-static Sflong_t ptr2int(const void *p) {
-  return (Sflong_t)(intptr_t)p;
+static long long ptr2int(const void *p) {
+  return (long long)(intptr_t)p;
 }
 
 static int iofread(void *chan, char *buf, int bufsize)
@@ -1595,7 +1595,7 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 
 #define MINTYPE (LAST_M+1)	/* First type occurs after last M_ */
 
-static char *typeName(int op) {
+static char *typeName(long op) {
     return typenames[op - MINTYPE];
 }
 
@@ -1656,11 +1656,9 @@ setval(Expr_t * pgm, Exnode_t * x, Exid_t * sym, Exref_t * ref,
 	    }
 	    break;
 	case V_tgtname:
-	    if (!streq(state->tgtname, v.string)) {
-		vmfree(pgm->vm, state->tgtname);
-		state->tgtname = vmstrdup(pgm->vm, v.string);
-		state->name_used = 0;
-	    }
+	    free(state->tgtname);
+	    state->tgtname = strdup(v.string);
+	    state->name_used = 0;
 	    break;
 	default:
 	    rv = -1;
@@ -2121,7 +2119,7 @@ static int stringOf(Expr_t *prog, Exnode_t *x, int arg) {
  * If arg is != 0, conversion unnecessary; just report possibility.
  * In particular, assume x != 0 if arg == 0.
  */
-static int convert(Exnode_t *x, int type, int arg) {
+static int convert(Exnode_t *x, long type, int arg) {
     Agobj_t *objp;
     int ret = -1;
 
@@ -2171,8 +2169,8 @@ static int convert(Exnode_t *x, int type, int arg) {
 	else if (validTVT(x->data.constant.value.integer))
 	    ret = 0;
 	else
-	    exerror("Integer value %" PRIdMAX " not legal for type tvtype_t",
-	            (intmax_t)x->data.constant.value.integer);
+	    exerror("Integer value %lld not legal for type tvtype_t",
+	            x->data.constant.value.integer);
     }
     /* in case libexpr hands us the trivial case */
     else if (x->type == type) {
@@ -2198,21 +2196,13 @@ static int convert(Exnode_t *x, int type, int arg) {
  * Calculate unique key for object.
  * We use this to unify local copies of nodes and edges.
  */
-static Extype_t keyval(Extype_t v, int type) {
+static Extype_t keyval(Extype_t v, long type) {
     if (type <= T_obj) {
 	v.integer = AGID(int2ptr(v.integer));
     }
     return v;
 }
 
-/* matchval:
- * Pattern match strings.
- */
-static int matchval(const char *str, const char *pat) {
-    return strgrpmatch(str, pat, NULL, 0,
-		       STR_MAXIMAL | STR_LEFT | STR_RIGHT);
-}
-
 /* a2t:
  * Convert type indices to symbolic name.
  */
@@ -2248,7 +2238,6 @@ static Exdisc_t *initDisc(Gpr_t * state)
     dp->getf = getval;
     dp->reff = refval;
     dp->setf = setval;
-    dp->matchf = matchval;
     dp->exitf = state->exitf;
     dp->types = a2t;
     dp->user = state;
