@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -25,10 +25,10 @@
 #include <cgraph/gv_ctype.h>
 #include <cgraph/ingraphs.h>
 #include <cgraph/exit.h>
+#include <cgraph/queue.h>
 #include <cgraph/stack.h>
 #include <common/globals.h>
 #include <gvpr/compile.h>
-#include <gvpr/queue.h>
 #include <gvpr/gvpr.h>
 #include <gvpr/actions.h>
 #include <ast/error.h>
@@ -526,14 +526,13 @@ static trav_fns REVfns = { agfstin, (nxttedgefn_t) agnxtin, 0, 0 };
 static void travBFS(Gpr_t * state, Expr_t* prog, comp_block * xprog)
 {
     nodestream nodes;
-    queue *q;
+    queue_t q = {0};
     ndata *nd;
     Agnode_t *n;
     Agedge_t *cure;
     Agedge_t *nxte;
     Agraph_t *g = state->curgraph;
 
-    q = mkQueue();
     nodes.oldroot = 0;
     nodes.prev = 0;
     while ((n = nextNode(state, &nodes))) {
@@ -541,8 +540,8 @@ static void travBFS(Gpr_t * state, Expr_t* prog, comp_block * xprog)
 	if (MARKED(nd))
 	    continue;
 	PUSH(nd, 0);
-	push(q, n);
-	while ((n = pull(q))) {
+	queue_push(&q, n);
+	while ((n = queue_pop(&q))) {
 	    nd = nData(n);
 	    MARK(nd);
 	    POP(nd);
@@ -555,14 +554,14 @@ static void travBFS(Gpr_t * state, Expr_t* prog, comp_block * xprog)
 		    continue;
 		if (!evalEdge(state, prog, xprog, cure)) continue;
 		if (!ONSTACK(nd)) {
-		    push(q, cure->node);
+		    queue_push(&q, cure->node);
 		    PUSH(nd,cure);
 		}
 	    }
 	}
     }
     state->tvedge = 0;
-    freeQ(q);
+    queue_free(&q);
 }
 
 static void travDFS(Gpr_t * state, Expr_t* prog, comp_block * xprog, trav_fns * fns)
@@ -935,7 +934,7 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
 	return 1;
     }
 
-    initGPRState(gs->state, gs->xprog->prog->vm);
+    initGPRState(gs->state);
     
     if ((uopts->flags & GV_USE_OUTGRAPH)) {
 	uopts->outgraphs = 0;
