@@ -10,7 +10,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -35,6 +35,7 @@
 #include <cgraph/bitarray.h>
 #include <cgraph/cgraph.h>
 #include <cgraph/gv_ctype.h>
+#include <cgraph/prisize_t.h>
 #include <cgraph/startswith.h>
 #include <cgraph/strcasecmp.h>
 #include <cgraph/streq.h>
@@ -113,7 +114,7 @@ bool user_pos(attrsym_t *posptr, attrsym_t *pinptr, node_t *np, int nG) {
 		ND_pinned(np) = P_PIN;
 	    return true;
 	} else
-	    agerr(AGERR, "node %s, position %s, expected two doubles\n",
+	    agerrorf("node %s, position %s, expected two doubles\n",
 		  agnameof(np), p);
     }
     return false;
@@ -189,33 +190,32 @@ static void set_label(void* obj, textlabel_t * l, char *name)
 }
 
 #ifdef IPSEPCOLA
-static cluster_data* cluster_map(graph_t *mastergraph, graph_t *g)
-{
+static cluster_data cluster_map(graph_t *mastergraph, graph_t *g) {
     graph_t *subg;
     node_t *n;
      /* array of arrays of node indices in each cluster */
     int **cs,*cn;
     int i,j,nclusters=0;
     bitarray_t assigned = bitarray_new(agnnodes(g));
-    cluster_data *cdata = gv_alloc(sizeof(cluster_data));
+    cluster_data cdata = {0};
 
-    cdata->ntoplevel = agnnodes(g);
+    cdata.ntoplevel = agnnodes(g);
     for (subg = agfstsubg(mastergraph); subg; subg = agnxtsubg(subg)) {
         if (startswith(agnameof(subg), "cluster")) {
             nclusters++;
         }
     }
-    cdata->nvars=0;
-    cdata->nclusters = nclusters;
-    cs = cdata->clusters = gv_calloc(nclusters, sizeof(int*));
-    cn = cdata->clustersizes = gv_calloc(nclusters, sizeof(int));
+    cdata.nvars=0;
+    cdata.nclusters = nclusters;
+    cs = cdata.clusters = gv_calloc(nclusters, sizeof(int*));
+    cn = cdata.clustersizes = gv_calloc(nclusters, sizeof(int));
     for (subg = agfstsubg(mastergraph); subg; subg = agnxtsubg(subg)) {
         /* clusters are processed by separate calls to ordered_edges */
         if (startswith(agnameof(subg), "cluster")) {
             int *c;
 
             *cn = agnnodes(subg);
-            cdata->nvars += *cn;
+            cdata.nvars += *cn;
             c = *cs++ = gv_calloc(*cn++, sizeof(int));
             for (n = agfstnode(subg); n; n = agnxtnode(subg, n)) {
                 node_t *gn;
@@ -226,31 +226,30 @@ static cluster_data* cluster_map(graph_t *mastergraph, graph_t *g)
                 }
                 *c++=ind;
                 bitarray_set(&assigned, ind, true);
-                cdata->ntoplevel--;
+                cdata.ntoplevel--;
             }
         }
     }
-    cdata->bb = gv_calloc(cdata->nclusters, sizeof(boxf));
-    cdata->toplevel = gv_calloc(cdata->ntoplevel, sizeof(int));
+    cdata.bb = gv_calloc(cdata.nclusters, sizeof(boxf));
+    cdata.toplevel = gv_calloc(cdata.ntoplevel, sizeof(int));
     for(i=j=0;i<agnnodes(g);i++) {
         if(!bitarray_get(assigned, i)) {
-            cdata->toplevel[j++]=i;
+            cdata.toplevel[j++] = i;
         }
     }
-    assert(cdata->ntoplevel==agnnodes(g)-cdata->nvars);
+    assert(cdata.ntoplevel == agnnodes(g) - cdata.nvars);
     bitarray_reset(&assigned);
     return cdata;
 }
 
-static void freeClusterData(cluster_data *c) {
-    if(c->nclusters>0) {
-        free(c->clusters[0]);
-        free(c->clusters);
-        free(c->clustersizes);
-        free(c->toplevel);
-        free(c->bb);
+static void freeClusterData(cluster_data c) {
+    if (c.nclusters > 0) {
+        free(c.clusters[0]);
+        free(c.clusters);
+        free(c.clustersizes);
+        free(c.toplevel);
+        free(c.bb);
     }
-    free(c);
 }
 #endif
 
@@ -263,7 +262,6 @@ static int user_spline(attrsym_t * E_pos, edge_t * e)
 {
     char *pos;
     int i, n, npts, nc;
-    pointf *ps = 0;
     pointf *pp;
     double x, y;
     int sflag = 0, eflag = 0;
@@ -303,18 +301,18 @@ static int user_spline(attrsym_t * E_pos, edge_t * e)
 	    gv_free_splines(e);
 	    if (!warned) {
 		warned = true;
-		agerr(AGWARN, "pos attribute for edge (%s,%s) doesn't have 3n+1 points\n", agnameof(agtail(e)), agnameof(aghead(e)));
+		agwarningf("pos attribute for edge (%s,%s) doesn't have 3n+1 points\n", agnameof(agtail(e)), agnameof(aghead(e)));
 	    }
 	    return 0;
 	}
-	ps = ALLOC(n, 0, pointf);
+	pointf *ps = gv_calloc(n, sizeof(pointf));
 	pp = ps;
 	while (n) {
 	    i = sscanf(pos, "%lf,%lf%n", &x, &y, &nc);
 	    if (i < 2) {
 		if (!warned) {
 		    warned = true;
-		    agerr(AGWARN, "syntax error in pos attribute for edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
+		    agwarningf("syntax error in pos attribute for edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
 		}
 		free(ps);
 		gv_free_splines(e);
@@ -541,7 +539,7 @@ int init_nop(Agraph_t * g, int adjust)
     attrsym_t *G_bb = agfindgraphattr(g, "bb");
     int didAdjust = 0;  /* Have nodes been moved? */
     int haveBackground;
-    bool translate = !mapBool(agget(g, "notranslate"), false);
+    bool translate = !mapbool(agget(g, "notranslate"));
 
     /* If G_bb not defined, define it */
     if (!G_bb)
@@ -550,7 +548,7 @@ int init_nop(Agraph_t * g, int adjust)
     scan_graph(g);		/* mainly to set up GD_neato_nlist */
     for (i = 0; (np = GD_neato_nlist(g)[i]); i++) {
 	if (!hasPos(np) && !startswith(agnameof(np), "cluster")) {
-	    agerr(AGERR, "node %s in graph %s has no position\n",
+	    agerrorf("node %s in graph %s has no position\n",
 		  agnameof(np), agnameof(g));
 	    return -1;
 	}
@@ -641,13 +639,13 @@ static int neatoModel(graph_t * g)
 	if (agattr(g, AGEDGE, "len", 0))
 	    return MODEL_MDS;
 	else {
-	    agerr(AGWARN,
+	    agwarningf(
 	        "edges in graph %s have no len attribute. Hence, the mds model\n", agnameof(g));
 	    agerr(AGPREV, "is inappropriate. Reverting to the shortest path model.\n");
 	    return MODEL_SHORTPATH;
 	}
     }
-    agerr(AGWARN,
+    agwarningf(
 	  "Unknown value %s for attribute \"model\" in graph %s - ignored\n",
 	  p, agnameof(g));
     return MODEL_SHORTPATH;
@@ -677,7 +675,7 @@ static int neatoMode(graph_t * g)
 #endif
 #endif
 	else
-	    agerr(AGWARN,
+	    agwarningf(
 		  "Illegal value %s for attribute \"mode\" in graph %s - ignored\n",
 		  str, agnameof(g));
     }
@@ -710,7 +708,7 @@ static void
 dfsCycle (vtx_data* graph, int i,int mode, node_t* nodes[])
 {
     node_t *np, *hp;
-    int j, e, f;
+    int j;
     /* if mode is IPSEP make it an in-edge
      * at both ends, so that an edge constraint won't be generated!
      */
@@ -719,12 +717,13 @@ dfsCycle (vtx_data* graph, int i,int mode, node_t* nodes[])
     np = nodes[i];
     ND_mark(np) = true;
     ND_onstack(np) = true;
-    for (e = 1; e < graph[i].nedges; e++) {
+    for (size_t e = 1; e < graph[i].nedges; e++) {
 	if (graph[i].edists[e] == 1.0) continue;  /* in edge */
 	j = graph[i].edges[e];
 	hp = nodes[j];
 	if (ND_onstack(hp)) {  /* back edge: reverse it */
             graph[i].edists[e] = x;
+            size_t f;
             for (f = 1; f < graph[j].nedges && graph[j].edges[f] != i; f++) ;
             assert (f < graph[j].nedges);
             graph[j].edists[f] = -1.0;
@@ -786,7 +785,7 @@ static vtx_data *makeGraphData(graph_t * g, int nv, int *nedges, int mode, int m
     float *edists = NULL;
 #endif
     PointMap *ps = newPM();
-    int i, i_nedges, idx;
+    int i, idx;
 
     /* lengths and weights unused in reweight model */
     bool haveLen = false;
@@ -833,7 +832,7 @@ static vtx_data *makeGraphData(graph_t * g, int nv, int *nedges, int mode, int m
 	else
 	    graph[i].edists = NULL;
 #endif
-	i_nedges = 1;		/* one for the self */
+	size_t i_nedges = 1; // one for the self
 
 	for (ep = agfstedge(g, np); ep; ep = agnxtedge(g, ep, np)) {
 	    if (aghead(ep) == agtail(ep))
@@ -843,8 +842,7 @@ static vtx_data *makeGraphData(graph_t * g, int nv, int *nedges, int mode, int m
 		if (haveWt)
 		    graph[i].eweights[idx] += ED_factor(ep);
 		if (haveLen) {
-		    int curlen = graph[i].ewgts[idx];
-		    graph[i].ewgts[idx] = MAX(ED_dist(ep), curlen);
+		    graph[i].ewgts[idx] = fmax(graph[i].ewgts[idx], ED_dist(ep));
 		}
 	    } else {
 		node_t *vp = agtail(ep) == np ? aghead(ep) : agtail(ep);
@@ -894,7 +892,7 @@ static vtx_data *makeGraphData(graph_t * g, int nv, int *nedges, int mode, int m
 	    eweights = gv_recalloc(graph[0].eweights, edges_size, 2 * ne + nv, sizeof(float));
 
 	for (i = 0; i < nv; i++) {
-	    int sz = graph[i].nedges;
+	    const size_t sz = graph[i].nedges;
 	    graph[i].edges = edges;
 	    edges += sz;
 	    if (haveLen) {
@@ -1000,7 +998,7 @@ static int checkExp (graph_t * G)
 {
     int exp = late_int(G, agfindgraphattr(G, exp_name), 2, 0);
     if (exp == 0 || exp > 2) {
-	agerr (AGWARN, "%s attribute value must be 1 or 2 - ignoring\n", exp_name);
+	agwarningf("%s attribute value must be 1 or 2 - ignoring\n", exp_name);
 	exp = 2;
     }
     return exp;
@@ -1024,7 +1022,7 @@ int checkStart(graph_t * G, int nG, int dflt)
     seed = 1;
     init = setSeed (G, dflt, &seed);
     if (N_pos && init != INIT_RANDOM) {
-	agerr(AGWARN, "node positions are ignored unless start=random\n");
+	agwarningf("node positions are ignored unless start=random\n");
     }
     if (init == INIT_REGULAR) initRegular(G, nG);
     srand48(seed);
@@ -1035,36 +1033,36 @@ int checkStart(graph_t * G, int nG, int dflt)
 void dumpData(graph_t * g, vtx_data * gp, int nv, int ne)
 {
     node_t *v;
-    int i, j, n;
+    int i;
 
     fprintf(stderr, "#nodes %d #edges %d\n", nv, ne);
     for (v = agfstnode(g); v; v = agnxtnode(g, v)) {
 	fprintf(stderr, "\"%s\" %d\n", agnameof(v), ND_id(v));
     }
     for (i = 0; i < nv; i++) {
-	n = gp[i].nedges;
-	fprintf(stderr, "[%d] %d\n", i, n);
-	for (j = 0; j < n; j++) {
+	const size_t n = gp[i].nedges;
+	fprintf(stderr, "[%d] %" PRISIZE_T "\n", i, n);
+	for (size_t j = 0; j < n; j++) {
 	    fprintf(stderr, "  %3d", gp[i].edges[j]);
 	}
 	fputs("\n", stderr);
 	if (gp[i].ewgts) {
 	    fputs("  ewgts", stderr);
-	    for (j = 0; j < n; j++) {
+	    for (size_t j = 0; j < n; j++) {
 		fprintf(stderr, "  %3f", gp[i].ewgts[j]);
 	    }
 	    fputs("\n", stderr);
 	}
 	if (gp[i].eweights) {
 	    fputs("  eweights", stderr);
-	    for (j = 0; j < n; j++) {
+	    for (size_t j = 0; j < n; j++) {
 		fprintf(stderr, "  %3f", gp[i].eweights[j]);
 	    }
 	    fputs("\n", stderr);
 	}
 	if (gp[i].edists) {
 	    fputs("  edists", stderr);
-	    for (j = 0; j < n; j++) {
+	    for (size_t j = 0; j < n; j++) {
 		fprintf(stderr, "  %3f", gp[i].edists[j]);
 	    }
 	    fputs("\n", stderr);
@@ -1164,7 +1162,7 @@ majorization(graph_t *mg, graph_t * g, int nv, int mode, int model, int dim, adj
 	else {
             char* str;
             ipsep_options opt;
-	    cluster_data *cs = cluster_map(mg,g);
+	    cluster_data cs = cluster_map(mg,g);
             pointf *nsize = gv_calloc(nv, sizeof(pointf));
             opt.edge_gap = lgap;
             opt.nsize = nsize;
@@ -1290,7 +1288,7 @@ static void kkNeato(Agraph_t * g, int nG, int model)
 	subset_model(g, nG);
     } else if (model == MODEL_CIRCUIT) {
 	if (!circuit_model(g, nG)) {
-	    agerr(AGWARN,
+	    agwarningf(
 		  "graph %s is disconnected. Hence, the circuit model\n",
 		  agnameof(g));
 	    agerr(AGPREV,
@@ -1410,7 +1408,7 @@ void neato_layout(Agraph_t * g)
 	}
 	else gv_postprocess(g, 0);
     } else {
-	bool noTranslate = mapBool(agget(g, "notranslate"), false);
+	bool noTranslate = mapbool(agget(g, "notranslate"));
 	PSinputscale = get_inputscale (g);
 	neato_init_graph(g);
 	layoutMode = neatoMode(g);
@@ -1431,15 +1429,14 @@ void neato_layout(Agraph_t * g)
 	if (Pack >= 0) {
 	    graph_t *gc;
 	    graph_t **cc;
-	    int n_cc;
-	    int i;
+	    size_t n_cc;
 	    bool pin;
 
 	    cc = pccomps(g, &n_cc, cc_pfx, &pin);
 
 	    if (n_cc > 1) {
 		bool *bp;
-		for (i = 0; i < n_cc; i++) {
+		for (size_t i = 0; i < n_cc; i++) {
 		    gc = cc[i];
 		    (void)graphviz_node_induce(gc, NULL);
 		    neatoLayout(g, gc, layoutMode, model, &am);
@@ -1469,7 +1466,7 @@ void neato_layout(Agraph_t * g)
 	    addZ (g);
 
 	    /* cleanup and remove component subgraphs */
-	    for (i = 0; i < n_cc; i++) {
+	    for (size_t i = 0; i < n_cc; i++) {
 		gc = cc[i];
 		free_scan_graph(gc);
 		agdelrec (gc, "Agraphinfo_t");
