@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -12,6 +12,7 @@
 #include "config.h"
 #include <cgraph/alloc.h>
 #include <cgraph/unreachable.h>
+#include <limits.h>
 #include <math.h>
 #include <neatogen/neato.h>
 #include <neatogen/adjust.h>
@@ -19,6 +20,7 @@
 #include <pathplan/vispath.h>
 #include <neatogen/multispline.h>
 #include <stdbool.h>
+#include <stddef.h>
 
 #ifdef ORTHO
 #include <ortho/ortho.h>
@@ -40,13 +42,11 @@ static bool swap_ends_p(edge_t * e)
 static splineInfo sinfo = {.swapEnds = swap_ends_p,
                            .splineMerge = spline_merge};
 
-static void
-make_barriers(Ppoly_t ** poly, int npoly, int pp, int qp,
-	      Pedge_t ** barriers, int *n_barriers)
-{
-    int i, j, k, n, b;
+static void make_barriers(Ppoly_t **poly, int npoly, int pp, int qp,
+                          Pedge_t **barriers, size_t *n_barriers) {
+    int i, j, k;
 
-    n = 0;
+    size_t n = 0;
     for (i = 0; i < npoly; i++) {
 	if (i == pp)
 	    continue;
@@ -55,15 +55,15 @@ make_barriers(Ppoly_t ** poly, int npoly, int pp, int qp,
 	n += poly[i]->pn;
     }
     Pedge_t *bar = gv_calloc(n, sizeof(Pedge_t));
-    b = 0;
+    size_t b = 0;
     for (i = 0; i < npoly; i++) {
 	if (i == pp)
 	    continue;
 	if (i == qp)
 	    continue;
-	for (j = 0; j < poly[i]->pn; j++) {
+	for (j = 0; j < (int)poly[i]->pn; j++) {
 	    k = j + 1;
-	    if (k >= poly[i]->pn)
+	    if (k >= (int)poly[i]->pn)
 		k = 0;
 	    bar[b].a = poly[i]->ps[j];
 	    bar[b].b = poly[i]->ps[k];
@@ -223,7 +223,8 @@ static edge_t *equivEdge(Dt_t * map, edge_t * e)
  */
 void makeSelfArcs(edge_t * e, int stepx)
 {
-    int cnt = ED_count(e);
+    assert(ED_count(e) >= 0);
+    const size_t cnt = (size_t)ED_count(e);
 
     if (cnt == 1 || Concentrate) {
 	edge_t *edges1[1];
@@ -232,15 +233,14 @@ void makeSelfArcs(edge_t * e, int stepx)
 	if (ED_label(e))
 	    updateBB(agraphof(agtail(e)), ED_label(e));
 	makePortLabels(e);
-    } else {
-	int i;
+    } else if (cnt > 1) {
 	edge_t **edges = gv_calloc(cnt, sizeof(edge_t*));
-	for (i = 0; i < cnt; i++) {
+	for (size_t i = 0; i < cnt; i++) {
 	    edges[i] = e;
 	    e = ED_to_virt(e);
 	}
 	makeSelfEdge(edges, 0, cnt, stepx, stepx, &sinfo);
-	for (i = 0; i < cnt; i++) {
+	for (size_t i = 0; i < cnt; i++) {
 	    e = edges[i];
 	    if (ED_label(e))
 		updateBB(agraphof(agtail(e)), ED_label(e));
@@ -250,6 +250,74 @@ void makeSelfArcs(edge_t * e, int stepx)
     }
 }
 
+/** calculate the slope of the tangent of an ellipse
+ *
+ * The equation for the slope `m` of the tangent of an ellipse as a function of
+ * `x` * is given by:
+ *
+ *           bx
+ * m = ± ――――――――――
+ *          _______
+ *       a √ a²- x²
+ *
+ * or
+ *
+ * m = ± (b * x) / (a * sqrt(a * a - x * x))
+ *
+ * We know that the slope is negative in the first and third quadrant, i.e.,
+ * when the signs of x and y are the same, so we use that to select the correct
+ * slope.
+ *
+ * @param a Half the width of the ellipse, i.e., the maximum x value
+ * @param b Half the height of the ellipse, i.e., the maximum y value
+ * @param p A point on the ellipse periphery in which to calculate the slope of
+ * the tangent
+ * @return The slope of the tangent in point `p`
+ */
+static double ellipse_tangent_slope(double a, double b, pointf p) {
+  assert(p.x != a &&
+         "cannot handle ellipse tangent slope in horizontal extreme point");
+  const double sign_y = p.y >= 0 ? 1 : -1;
+  const double m = -sign_y * (b * p.x) / (a * sqrt(a * a - p.x * p.x));
+  assert(isfinite(m) && "ellipse tangent slope is infinite");
+  return m;
+}
+
+/** calculate the intersection of two lines
+ *
+ * @param l0 First line
+ * @param l1 Second line
+ * @return Intersection of the two lines
+ */
+static pointf line_intersection(linef l0, linef l1) {
+  const double x =
+      (l0.m * l0.p.x - l0.p.y - l1.m * l1.p.x + l1.p.y) / (l0.m - l1.m);
+  const double y = l0.p.y + l0.m * (x - l0.p.x);
+  return (pointf){x, y};
+}
+
+/** calculate a corner of a polygon circumscribed about an ellipse
+ *
+ * @param a Half the width of the ellipse, i.e., the maximum x value
+ * @param b Half the height of the ellipse, i.e., the maximum y value
+ * @param i Index of the polygon corner
+ * @param nsides Number of sides of the polygon
+ * @return Polygon corner at index `i`
+ */
+static pointf circumscribed_polygon_corner_about_ellipse(double a, double b,
+                                                         size_t i,
+                                                         size_t nsides) {
+  const double angle0 = 2.0 * M_PI * ((double)i - 0.5) / (double)nsides;
+  const double angle1 = 2.0 * M_PI * ((double)i + 0.5) / (double)nsides;
+  const pointf p0 = {a * cos(angle0), b * sin(angle0)};
+  const pointf p1 = {a * cos(angle1), b * sin(angle1)};
+  const double m0 = ellipse_tangent_slope(a, b, p0);
+  const double m1 = ellipse_tangent_slope(a, b, p1);
+  const linef line0 = {{p0.x, p0.y}, m0};
+  const linef line1 = {{p1.x, p1.y}, m1};
+  return line_intersection(line0, line1);
+}
+
 /* makeObstacle:
  * Given a node, return an obstacle reflecting the
  * node's geometry. pmargin specifies how much space to allow
@@ -266,7 +334,6 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, bool isOrtho)
 {
     Ppoly_t *obs;
     polygon_t *poly;
-    double adj = 0.0;
     size_t sides;
     pointf polyp;
     boxf b;
@@ -299,29 +366,39 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, bool isOrtho)
 		vs[3].x = b.LL.x;
 		vs[3].y = b.UR.y;
 	    } else {
-		p.x = -ND_lw(n);
-		p.y = -ND_ht(n)/2.0;
+		const double width = ND_lw(n) + ND_rw(n);
+		const double outline_width = INCH2PS(ND_outline_width(n));
+		// scale lw and rw proportionally to sum to outline width
+		const double outline_lw = ND_lw(n) * outline_width / width;
+		const double outline_ht = INCH2PS(ND_outline_height(n));
+		p.x = -outline_lw;
+		p.y = -outline_ht / 2.0;
 		vs[0] = p;
-		p.x = ND_lw(n);
+		p.x = outline_lw;
 		vs[1] = p;
-		p.y = ND_ht(n)/2.0;
+		p.y = outline_ht / 2.0;
 		vs[2] = p;
-		p.x = -ND_lw(n);
+		p.x = -outline_lw;
 		vs[3] = p;
 	    }
 	}
 	else if (poly->sides >= 3) {
 	    isPoly = true;
 	    sides = poly->sides;
-	    verts = poly->vertices;
-	    margin.x = pmargin->x;
-	    margin.y = pmargin->y;
+            const double penwidth = late_double(n, N_penwidth, 1.0, 0.0);
+            // possibly use extra vertices representing the outline, i.e., the
+            // outermost periphery with penwidth taken into account
+            const size_t extra_peripheries = poly->peripheries >= 1 && penwidth > 0.0 ? 1 : 0;
+            const size_t outline_periphery = poly->peripheries + extra_peripheries;
+            const size_t vertices_offset = outline_periphery >= 1 ? (outline_periphery - 1) * sides : 0;
+            verts = poly->vertices + vertices_offset;
+            margin.x = pmargin->x;
+            margin.y = pmargin->y;
 	} else {		/* ellipse */
 	    isPoly = false;
 	    sides = 8;
-	    adj = drand48() * .01;
 	}
-	obs->pn = (int)sides;
+	obs->pn = sides;
 	obs->ps = gv_calloc(sides, sizeof(Ppoint_t));
 	/* assuming polys are in CCW order, and pathplan needs CW */
 	for (size_t j = 0; j < sides; j++) {
@@ -363,17 +440,12 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, bool isOrtho)
 		    polyp.y = verts[j].y * margin.y;
 		}
 	    } else {
-		double c, s;
-		c = cos(2.0 * M_PI * (double)j / (double)sides + adj);
-		s = sin(2.0 * M_PI * (double)j / (double)sides + adj);
-		if (pmargin->doAdd) {
-		    polyp.x =  c*(ND_lw(n)+ND_rw(n)+pmargin->x) / 2.0;
-		    polyp.y =  s*(ND_ht(n)+pmargin->y) / 2.0;
-		}
-		else {
-		    polyp.x = pmargin->x * c * (ND_lw(n) + ND_rw(n)) / 2.0;
-		    polyp.y = pmargin->y * s * ND_ht(n) / 2.0;
-		}
+		const double width = INCH2PS(ND_outline_width(n));
+		const double height = INCH2PS(ND_outline_height(n));
+		margin = pmargin->doAdd ? (pointf) {pmargin->x, pmargin->y} : (pointf) {0.0, 0.0};
+		const double ellipse_a = (width + margin.x) / 2.0;
+		const double ellipse_b = (height + margin.y) / 2.0;
+		polyp = circumscribed_polygon_corner_about_ellipse(ellipse_a, ellipse_b, j, sides);
 	    }
 	    obs->ps[sides - j - 1].x = polyp.x + ND_coord(n).x;
 	    obs->ps[sides - j - 1].y = polyp.y + ND_coord(n).y;
@@ -458,8 +530,7 @@ static void makePolyline(edge_t * e) {
     make_polyline (line, &spl);
     if (Verbose > 1)
 	fprintf(stderr, "polyline %s %s\n", agnameof(agtail(e)), agnameof(aghead(e)));
-    assert(spl.pn >= 0);
-    clip_and_install(e, aghead(e), spl.ps, (size_t)spl.pn, &sinfo);
+    clip_and_install(e, aghead(e), spl.ps, spl.pn, &sinfo);
     addEdgeLabels(e);
 }
 
@@ -476,7 +547,7 @@ static void makePolyline(edge_t * e) {
 void makeSpline(edge_t *e, Ppoly_t **obs, int npoly, bool chkPts) {
     Ppolyline_t line, spline;
     Pvector_t slopes[2];
-    int i, n_barriers;
+    int i;
     int pp, qp;
     Ppoint_t p, q;
     Pedge_t *barriers;
@@ -494,19 +565,19 @@ void makeSpline(edge_t *e, Ppoly_t **obs, int npoly, bool chkPts) {
 		qp = i;
 	}
 
+    size_t n_barriers;
     make_barriers(obs, npoly, pp, qp, &barriers, &n_barriers);
     slopes[0].x = slopes[0].y = 0.0;
     slopes[1].x = slopes[1].y = 0.0;
     if (Proutespline(barriers, n_barriers, line, slopes, &spline) < 0) {
-	agerr (AGERR, "makeSpline: failed to make spline edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
+	agerrorf("makeSpline: failed to make spline edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
 	return;
     }
 
     /* north why did you ever use int coords */
     if (Verbose > 1)
 	fprintf(stderr, "spline %s %s\n", agnameof(agtail(e)), agnameof(aghead(e)));
-    assert(spline.pn >= 0);
-    clip_and_install(e, aghead(e), spline.ps, (size_t)spline.pn, &sinfo);
+    clip_and_install(e, aghead(e), spline.ps, spline.pn, &sinfo);
     free(barriers);
     addEdgeLabels(e);
 }
@@ -562,9 +633,9 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
 	}
 	else {
 	    if (edgetype == EDGETYPE_ORTHO)
-		agerr(AGWARN, "the bounding boxes of some nodes touch - falling back to straight line edges\n");
+		agwarningf("the bounding boxes of some nodes touch - falling back to straight line edges\n");
 	    else 
-		agerr(AGWARN, "some nodes with margin (%.02f,%.02f) touch - falling back to straight line edges\n", pmargin->x, pmargin->y);
+		agwarningf("some nodes with margin (%.02f,%.02f) touch - falling back to straight line edges\n", pmargin->x, pmargin->y);
 	}
     }
 
@@ -584,7 +655,7 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
     }
 #ifdef ORTHO
     else if (legal && edgetype == EDGETYPE_ORTHO) {
-	orthoEdges (g, 0);
+	orthoEdges(g, false);
 	useEdges = 1;
     }
 #endif
@@ -740,7 +811,7 @@ void spline_edges0(graph_t *g, bool set_aspect) {
     if (et == EDGETYPE_NONE) return;
 #ifndef ORTHO
     if (et == EDGETYPE_ORTHO) {
-	agerr (AGWARN, "Orthogonal edges not yet supported\n");
+	agwarningf("Orthogonal edges not yet supported\n");
 	et = EDGETYPE_PLINE;
 	GD_flags(g->root) &= ~EDGETYPE_ORTHO;
 	GD_flags(g->root) |= EDGETYPE_PLINE;
