@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -664,7 +664,7 @@ void remove_overlap(int dim, SparseMatrix A, double *x, double *label_sizes, int
 
     if (once == 0) {
 	once = 1;
-	agerr(AGERR, "remove_overlap: Graphviz not built with triangulation library\n");
+	agerrorf("remove_overlap: Graphviz not built with triangulation library\n");
     }
 }
 #endif
