@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -139,8 +139,6 @@ setSeg (segment* sp, bool dir, double fix, double b1, double b2, int l1, int l2)
  * the two dummy nodes representing the center of the two real nodes,
  * and the two nodes on the boundary of the two real nodes.
  */
-#define PUSH(rte,P) (rte.p[rte.n++] = P)
-
 static route
 convertSPtoRoute (sgraph* g, snode* fst, snode* lst)
 {
@@ -258,7 +256,7 @@ static void freeChannel(channel *cp, Dtdisc_t *disc) {
     (void)disc;
 
     free_graph (cp->G);
-    free (cp->seg_list);
+    seg_list_free(&cp->seg_list);
     free (cp);
 }
 
@@ -415,9 +413,8 @@ extractVChans (maze* mp)
 static void
 insertChan (channel* chan, segment* seg)
 {
-    seg->ind_no = chan->cnt++;
-    chan->seg_list = ALLOC(chan->cnt, chan->seg_list, segment*);
-    chan->seg_list[chan->cnt-1] = seg;
+    seg->ind_no = seg_list_size(&chan->seg_list);
+    seg_list_append(&chan->seg_list, seg);
 }
 
 static channel*
@@ -533,20 +530,19 @@ static void putSeg (FILE* fp, segment* seg)
 }
 
 static DEBUG_FN void dumpChanG(channel *cp, int v) {
-  int k;
   intitem* ip;
   Dt_t* adj;
 
-  if (cp->cnt < 2) return;
+  if (seg_list_size(&cp->seg_list) < 2) return;
   fprintf (stderr, "channel %d (%f,%f)\n", v, cp->p.p1, cp->p.p2);
-  for (k=0;k<cp->cnt;k++) {
+  for (size_t k = 0; k < seg_list_size(&cp->seg_list); ++k) {
     adj = cp->G->vertices[k].adj_list;
     if (dtsize(adj) == 0) continue;
-    putSeg (stderr, cp->seg_list[k]);
+    putSeg(stderr, seg_list_get(&cp->seg_list, k));
     fputs (" ->\n", stderr);
     for (ip = dtfirst(adj); ip; ip = dtnext(adj, ip)) {
       fputs ("     ", stderr);
-      putSeg (stderr, cp->seg_list[ip->id]);
+      putSeg(stderr, seg_list_get(&cp->seg_list, ip->id));
       fputs ("\n", stderr);
     }
   }
@@ -559,19 +555,18 @@ assignTrackNo (Dt_t* chans)
     Dtlink_t* l1;
     Dtlink_t* l2;
     channel* cp;
-    int k;
 
     for (l1 = dtflatten (chans); l1; l1 = dtlink(chans,l1)) {
 	lp = ((chanItem*)l1)->chans;
 	for (l2 = dtflatten (lp); l2; l2 = dtlink(lp,l2)) {
 	    cp = (channel*)l2;
-	    if (cp->cnt) {
+	    if (!seg_list_is_empty(&cp->seg_list)) {
 #ifdef DEBUG
     if (odb_flags & ODB_CHANG) dumpChanG (cp, ((chanItem*)l1)->v);
 #endif
 		top_sort (cp->G);
-		for (k=0;k<cp->cnt;k++)
-		    cp->seg_list[k]->track_no = cp->G->vertices[k].topsort_order+1;
+		for (size_t k = 0; k < seg_list_size(&cp->seg_list); ++k)
+		    seg_list_get(&cp->seg_list, k)->track_no = cp->G->vertices[k].topsort_order+1;
 	    }
    	}
     }
@@ -589,7 +584,7 @@ create_graphs(Dt_t* chans)
 	lp = ((chanItem*)l1)->chans;
 	for (l2 = dtflatten (lp); l2; l2 = dtlink(lp,l2)) {
 	    cp = (channel*)l2;
-	    cp->G = make_graph (cp->cnt);
+	    cp->G = make_graph(seg_list_size(&cp->seg_list));
    	}
     }
 }
@@ -714,7 +709,7 @@ static int
 seg_cmp(segment* S1, segment* S2)		
 {
     if(S1->isVert!=S2->isVert||S1->comm_coord!=S2->comm_coord) {
-	agerr (AGERR, "incomparable segments !! -- Aborting\n");
+	agerrorf("incomparable segments !! -- Aborting\n");
 	return -2;
     }
     if(S1->isVert)
@@ -726,14 +721,14 @@ seg_cmp(segment* S1, segment* S2)
 static int
 add_edges_in_G(channel* cp)
 {
-    int x,y;
-    segment** seg_list = cp->seg_list;
-    int size = cp->cnt;
+    seg_list_t *seg_list = &cp->seg_list;
+    const size_t size = seg_list_size(&cp->seg_list);
     rawgraph* G = cp->G;
 
-    for(x=0;x+1<size;x++) {
-	for(y=x+1;y<size;y++) {
-	    int cmp = seg_cmp(seg_list[x],seg_list[y]);
+    for (size_t x = 0; x + 1 < size; ++x) {
+	for (size_t y = x + 1; y < size; ++y) {
+	    const int cmp = seg_cmp(seg_list_get(seg_list, x),
+	                            seg_list_get(seg_list, y));
 	    if (cmp == -2) {
 		return -1;
 	    } else if (cmp > 0) {
@@ -759,7 +754,7 @@ add_np_edges (Dt_t* chans)
 	lp = ((chanItem*)l1)->chans;
 	for (l2 = dtflatten (lp); l2; l2 = dtlink(lp,l2)) {
 	    cp = (channel*)l2;
-	    if (cp->cnt)
+	    if (!seg_list_is_empty(&cp->seg_list))
 		if (add_edges_in_G(cp)) {
 		  return -1;
 		}
@@ -975,7 +970,6 @@ removeEdge(segment* seg1, segment* seg2, int dir, maze* mp)
 static int
 addPEdges (channel* cp, maze* mp)
 {
-    int i,j;
     /* dir[1,2] are used to figure out whether we should use prev 
      * pointers or next pointers -- 0 : decrease, 1 : increase
      */
@@ -988,61 +982,74 @@ addPEdges (channel* cp, maze* mp)
     int prec1, prec2;
     pair p;
     rawgraph* G = cp->G;
-    segment** segs = cp->seg_list;
+    seg_list_t *segs = &cp->seg_list;
 
-    for(i=0;i+1<cp->cnt;i++) {
-	for(j=i+1;j<cp->cnt;j++) {
+    for(size_t i = 0; i + 1 < seg_list_size(&cp->seg_list); ++i) {
+	for(size_t j = i + 1; j < seg_list_size(&cp->seg_list); ++j) {
 	    if (!edge_exists(G,i,j) && !edge_exists(G,j,i)) {
-		if (is_parallel(segs[i], segs[j])) {
+		if (is_parallel(seg_list_get(segs, i), seg_list_get(segs, j))) {
 		/* get_directions */
-		    if(segs[i]->prev==0) {
-			if(segs[j]->prev==0)
+		    if (seg_list_get(segs, i)->prev == 0) {
+			if (seg_list_get(segs, j)->prev == 0)
 			    dir = 0;
 			else
 			    dir = 1;
 		    }
-		    else if(segs[j]->prev==0) {
+		    else if (seg_list_get(segs, j)->prev == 0) {
 			dir = 1;
 		    }
 		    else {
-			if(segs[i]->prev->comm_coord==segs[j]->prev->comm_coord)
+			if (seg_list_get(segs, i)->prev->comm_coord ==
+			    seg_list_get(segs, j)->prev->comm_coord)
 			    dir = 0;
 			else
 			    dir = 1;
 		    }
 
-		    if (decide_point(&p, segs[i], segs[j], 0, dir) != 0) {
+		    if (decide_point(&p, seg_list_get(segs, i), seg_list_get(segs, j), 0, dir)
+		        != 0) {
 			return -1;
 		    }
 		    hops.a = p.a;
 		    prec1 = p.b;
-		    if (decide_point(&p, segs[i], segs[j], 1, 1-dir) != 0) {
+		    if (decide_point(&p, seg_list_get(segs, i), seg_list_get(segs, j), 1,
+		                     1 - dir) != 0) {
 			return -1;
 		    }
 		    hops.b = p.a;
 		    prec2 = p.b;
 
 		    if (prec1 == -1) {
-			set_parallel_edges (segs[j], segs[i], dir, 0, hops.a, mp);
-			set_parallel_edges (segs[j], segs[i], 1-dir, 1, hops.b, mp);
+			set_parallel_edges(seg_list_get(segs, j), seg_list_get(segs, i), dir, 0,
+			                   hops.a, mp);
+			set_parallel_edges(seg_list_get(segs, j), seg_list_get(segs, i), 1 - dir, 1,
+			                   hops.b, mp);
 			if(prec2==1)
-			    removeEdge (segs[i], segs[j], 1-dir, mp);
+			    removeEdge(seg_list_get(segs, i), seg_list_get(segs, j), 1 - dir, mp);
 		    } else if (prec1 == 0) {
 			if (prec2 == -1) {
-			    set_parallel_edges (segs[j], segs[i], dir, 0, hops.a, mp);
-			    set_parallel_edges (segs[j], segs[i], 1-dir, 1, hops.b, mp);
+			    set_parallel_edges(seg_list_get(segs, j), seg_list_get(segs, i), dir, 0,
+			                       hops.a, mp);
+			    set_parallel_edges(seg_list_get(segs, j), seg_list_get(segs, i), 1 - dir,
+			                       1, hops.b, mp);
 			} else if (prec2 == 0) {
-			    set_parallel_edges (segs[i], segs[j], 0, dir, hops.a, mp);
-			    set_parallel_edges (segs[i], segs[j], 1, 1-dir, hops.b, mp);
+			    set_parallel_edges(seg_list_get(segs, i), seg_list_get(segs, j), 0, dir,
+			                       hops.a, mp);
+			    set_parallel_edges(seg_list_get(segs, i), seg_list_get(segs, j), 1,
+			                       1 - dir, hops.b, mp);
 			} else if (prec2 == 1) {
-			    set_parallel_edges (segs[i], segs[j], 0, dir, hops.a, mp);
-			    set_parallel_edges (segs[i], segs[j], 1, 1-dir, hops.b, mp);
+			    set_parallel_edges(seg_list_get(segs, i), seg_list_get(segs, j), 0, dir,
+			                       hops.a, mp);
+			    set_parallel_edges(seg_list_get(segs, i), seg_list_get(segs, j), 1,
+			                       1 - dir, hops.b, mp);
 			}
 		    } else if (prec1 == 1) {
-			set_parallel_edges (segs[i], segs[j], 0, dir, hops.a, mp);
-			set_parallel_edges (segs[i], segs[j], 1, 1-dir, hops.b, mp);
+			set_parallel_edges(seg_list_get(segs, i), seg_list_get(segs, j), 0, dir,
+			                   hops.a, mp);
+			set_parallel_edges(seg_list_get(segs, i), seg_list_get(segs, j), 1, 1 - dir,
+			                   hops.b, mp);
 			if(prec2==-1)
-			    removeEdge (segs[i], segs[j], 1-dir, mp);
+			    removeEdge(seg_list_get(segs, i), seg_list_get(segs, j), 1 - dir, mp);
 		    }
 		}
 	    }
@@ -1105,7 +1112,7 @@ static double
 vtrack (segment* seg, maze* m)
 {
   channel* chp = chanSearch(m->vchans, seg);
-  double f = ((double)seg->track_no)/(chp->cnt+1); 
+  const double f = (double)seg->track_no / ((double)seg_list_size(&chp->seg_list) + 1);
   double left = chp->cp->bb.LL.x;
   double right = chp->cp->bb.UR.x;
   return left + f*(right-left);
@@ -1115,7 +1122,7 @@ static int
 htrack (segment* seg, maze* m)
 {
   channel* chp = chanSearch(m->hchans, seg);
-  double f = 1.0 - ((double)seg->track_no)/(chp->cnt+1); 
+  double f = 1.0 - (double)seg->track_no / ((double)seg_list_size(&chp->seg_list) + 1);
   double lo = chp->cp->bb.LL.y;
   double hi = chp->cp->bb.UR.y;
   return lo + f*(hi-lo);
@@ -1129,9 +1136,8 @@ addPoints(pointf p0, pointf p1)
     return p0;
 }
 
-static void
-attachOrthoEdges(maze* mp, size_t n_edges, route* route_list, splineInfo *sinfo, epair_t es[], int doLbls)
-{
+static void attachOrthoEdges(maze *mp, size_t n_edges, route* route_list,
+                             splineInfo *sinfo, epair_t es[], bool doLbls) {
     int ipt;
     pointf* ispline = 0;
     size_t splsz = 0;
@@ -1202,9 +1208,16 @@ edgeLen (Agedge_t* e)
     return (int)DIST2(p,q);
 }
 
-static int edgecmp(epair_t* e0, epair_t* e1)
-{
-    return e0->d - e1->d;
+static int edgecmp(const void *x, const void *y) {
+  const epair_t *e0 = x;
+  const epair_t *e1 = y;
+  if (e0->d > e1->d) {
+    return 1;
+  }
+  if (e0->d < e1->d) {
+    return -1;
+  }
+  return 0;
 }
 
 static bool spline_merge(node_t * n)
@@ -1223,12 +1236,10 @@ static splineInfo sinfo = { swap_ends_p, spline_merge, true, true };
 
 /* orthoEdges:
  * For edges without position information, construct an orthogonal routing.
- * If doLbls is true, use edge label info when available to guide routing, 
+ * If useLbls is true, use edge label info when available to guide routing, 
  * and set label pos for those edges for which this info is not available.
  */
-void
-orthoEdges (Agraph_t* g, int doLbls)
-{
+void orthoEdges(Agraph_t *g, bool useLbls) {
     sgraph* sg;
     maze* mp;
     route* route_list;
@@ -1269,14 +1280,16 @@ orthoEdges (Agraph_t* g, int doLbls)
 		case 's' :
 		    odb_flags |= ODB_SGRAPH;    // emit search graph 
 		    break;
+		default:
+		    break;
 		}
 	    }
 	}
     }
 #endif
-    if (doLbls) {
-	agerr(AGWARN, "Orthogonal edges do not currently handle edge labels. Try using xlabels.\n");
-	doLbls = 0;
+    if (useLbls) {
+	agwarningf("Orthogonal edges do not currently handle edge labels. Try using xlabels.\n");
+	useLbls = false;
     }
     mp = mkMaze(g);
     sg = mp->sg;
@@ -1309,7 +1322,7 @@ orthoEdges (Agraph_t* g, int doLbls)
 
     route_list = gv_calloc(n_edges, sizeof(route));
 
-    qsort(es, n_edges, sizeof(epair_t), (qsort_cmpf) edgecmp);
+    qsort(es, n_edges, sizeof(epair_t), edgecmp);
 
     gstart = sg->nnodes;
     PQgen (sg->nnodes+2);
@@ -1323,7 +1336,7 @@ orthoEdges (Agraph_t* g, int doLbls)
         start = CELL(agtail(e));
         dest = CELL(aghead(e));
 
-	if (doLbls && (lbl = ED_label(e)) && lbl->set) {
+	if (useLbls && (lbl = ED_label(e)) && lbl->set) {
 	}
 	else {
 	    if (start == dest)
@@ -1348,7 +1361,7 @@ orthoEdges (Agraph_t* g, int doLbls)
 #ifdef DEBUG
     if (odb_flags & ODB_ROUTE) emitGraph (stderr, mp, n_edges, route_list, es);
 #endif
-    attachOrthoEdges(mp, n_edges, route_list, &sinfo, es, doLbls);
+    attachOrthoEdges(mp, n_edges, route_list, &sinfo, es, useLbls);
 
 orthofinish:
     if (Concentrate)
@@ -1422,7 +1435,7 @@ coordOf (cell* cp, snode* np)
 	p.x = cp->bb.UR.x;
     }
     else {
-	agerr (AGERR, "Node not adjacent to cell -- Aborting\n");
+	agerrorf("Node not adjacent to cell -- Aborting\n");
 	graphviz_exit(EXIT_FAILURE);
     }
     return p;
@@ -1523,17 +1536,15 @@ static DEBUG_FN void emitSearchGraph(FILE *fp, sgraph *sg) {
 
 static DEBUG_FN void emitGraph(FILE *fp, maze *mp, size_t n_edges,
                                route *route_list, epair_t es[]) {
-    boxf bb, absbb;
-
-    absbb.LL.x = absbb.LL.y = MAXDOUBLE;
-    absbb.UR.x = absbb.UR.y = -MAXDOUBLE;
+    boxf absbb = {.LL = {.x = MAXDOUBLE, .y = MAXDOUBLE},
+                  .UR = {.x = -MAXDOUBLE, .y = -MAXDOUBLE}};
 
     fprintf (fp, "%s", prolog2);
     fprintf (fp, "%d %d translate\n", TRANS, TRANS);
 
     fputs ("0 0 1 setrgbcolor\n", fp);
     for (int i = 0; i < mp->ngcells; i++) {
-      bb = mp->gcells[i].bb;
+      const boxf bb = mp->gcells[i].bb;
       fprintf (fp, "%f %f %f %f node\n", bb.LL.x, bb.LL.y, bb.UR.x, bb.UR.y);
     }
 
@@ -1543,7 +1554,7 @@ static DEBUG_FN void emitGraph(FILE *fp, maze *mp, size_t n_edges,
     
     fputs ("0.8 0.8 0.8 setrgbcolor\n", fp);
     for (int i = 0; i < mp->ncells; i++) {
-      bb = mp->cells[i].bb;
+      const boxf bb = mp->cells[i].bb;
       fprintf (fp, "%f %f %f %f cell\n", bb.LL.x, bb.LL.y, bb.UR.x, bb.UR.y);
       absbb.LL.x = MIN(absbb.LL.x, bb.LL.x);
       absbb.LL.y = MIN(absbb.LL.y, bb.LL.y);
@@ -1551,10 +1562,10 @@ static DEBUG_FN void emitGraph(FILE *fp, maze *mp, size_t n_edges,
       absbb.UR.y = MAX(absbb.UR.y, bb.UR.y);
     }
 
-    boxf bbox;
-    bbox.LL.x = absbb.LL.x + TRANS;
-    bbox.LL.y = absbb.LL.y + TRANS;
-    bbox.UR.x = absbb.UR.x + TRANS;
-    bbox.UR.y = absbb.UR.y + TRANS;
+    const boxf bbox = {
+      .LL = {.x = absbb.LL.x + TRANS,
+             .y = absbb.LL.y + TRANS},
+      .UR = {.x = absbb.UR.x + TRANS,
+             .y = absbb.UR.y + TRANS}};
     fprintf (fp, epilog2, bbox.LL.x, bbox.LL.y,  bbox.UR.x, bbox.UR.y);
 }
