@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -21,18 +21,29 @@
 #include <pack/pack.h>
 #include <stdbool.h>
 
-#define MARKED(stk,n) ((stk)->markfn(n,-1))
-#define MARK(stk,n)   ((stk)->markfn(n,1))
-#define UNMARK(stk,n) ((stk)->markfn(n,0))
-
 typedef struct {
     gv_stack_t data;
     void (*actionfn) (Agnode_t *, void *);
-    int (*markfn) (Agnode_t *, int);
+    bool (*markfn)(Agnode_t *, int);
 } stk_t;
 
+/// does `n` have a mark set?
+static bool marked(const stk_t *stk, Agnode_t *n) {
+  return stk->markfn(n, -1);
+}
+
+/// set a mark on `n`
+static void mark(const stk_t *stk, Agnode_t *n) {
+  stk->markfn(n, 1);
+}
+
+/// unset a mark on `n`
+static void unmark(const stk_t *stk, Agnode_t *n) {
+  stk->markfn(n, 0);
+}
+
 static void initStk(stk_t *sp, void (*actionfn)(Agnode_t*, void*),
-     int (*markfn) (Agnode_t *, int))
+                    bool (*markfn)(Agnode_t *, int))
 {
     sp->data = (gv_stack_t){0};
     sp->actionfn = actionfn;
@@ -45,7 +56,7 @@ static void freeStk (stk_t* sp)
 }
 
 static void push(stk_t *sp, Agnode_t *np) {
-  MARK(sp, np);
+  mark(sp, np);
   stack_push(&sp->data, np);
 }
 
@@ -72,7 +83,7 @@ static size_t dfs(Agraph_t * g, Agnode_t * n, void *state, stk_t* stk)
         for (e = agfstedge(g, n); e; e = agnxtedge(g, e, n)) {
 	    if ((other = agtail(e)) == n)
 		other = aghead(e);
-            if (!MARKED(stk,other))
+            if (!marked(stk, other))
                 push(stk, other);
         }
     }
@@ -95,13 +106,11 @@ static void insertFn(Agnode_t * n, void *state)
     agsubnode(state, n, 1);
 }
 
-static int markFn (Agnode_t* n, int v)
-{
-    int ret;
-    if (v < 0) return ND_mark(n);
-    ret = ND_mark(n);
-    ND_mark(n) = (char) v;
-    return ret;
+static bool markFn(Agnode_t *n, int v) {
+    if (v < 0) return ND_mark(n) != 0;
+    const size_t ret = ND_mark(n);
+    ND_mark(n) = v != 0;
+    return ret != 0;
 }
 
 static void setPrefix(agxbuf *xb, const char *pfx) {
@@ -123,8 +132,7 @@ static void setPrefix(agxbuf *xb, const char *pfx) {
  * be obtained from the root graph.
  * Return NULL if graph is empty.
  */
-Agraph_t **pccomps(Agraph_t * g, int *ncc, char *pfx, bool *pinned)
-{
+Agraph_t **pccomps(Agraph_t *g, size_t *ncc, char *pfx, bool *pinned) {
     size_t c_cnt = 0;
     agxbuf name = {0};
     Agraph_t *out = NULL;
@@ -142,11 +150,11 @@ Agraph_t **pccomps(Agraph_t * g, int *ncc, char *pfx, bool *pinned)
 
     initStk(&stk, insertFn, markFn);
     for (n = agfstnode(g); n; n = agnxtnode(g, n))
-	UNMARK(&stk,n);
+	unmark(&stk, n);
 
     /* Component with pinned nodes */
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	if (MARKED(&stk,n) || !isPinned(n))
+	if (marked(&stk, n) || !isPinned(n))
 	    continue;
 	if (!out) {
 	    setPrefix(&name, pfx);
@@ -162,7 +170,7 @@ Agraph_t **pccomps(Agraph_t * g, int *ncc, char *pfx, bool *pinned)
 
     /* Remaining nodes */
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	if (MARKED(&stk,n))
+	if (marked(&stk, n))
 	    continue;
 	setPrefix(&name, pfx);
 	agxbprint(&name, "%" PRISIZE_T, c_cnt);
@@ -179,7 +187,7 @@ Agraph_t **pccomps(Agraph_t * g, int *ncc, char *pfx, bool *pinned)
     freeStk (&stk);
     agxbfree(&name);
     ccs = gv_recalloc(ccs, bnd, c_cnt, sizeof(Agraph_t*));
-    *ncc = (int) c_cnt;
+    *ncc = c_cnt;
     *pinned = pin;
     return ccs;
 }
@@ -193,8 +201,7 @@ Agraph_t **pccomps(Agraph_t * g, int *ncc, char *pfx, bool *pinned)
  * be obtained from the root graph.
  * Returns NULL on error or if graph is empty.
  */
-Agraph_t **ccomps(Agraph_t * g, int *ncc, char *pfx)
-{
+Agraph_t **ccomps(Agraph_t *g, size_t *ncc, char *pfx) {
     size_t c_cnt = 0;
     agxbuf name = {0};
     Agraph_t *out;
@@ -210,10 +217,10 @@ Agraph_t **ccomps(Agraph_t * g, int *ncc, char *pfx)
     Agraph_t **ccs = gv_calloc(bnd, sizeof(Agraph_t*));
     initStk(&stk, insertFn, markFn);
     for (n = agfstnode(g); n; n = agnxtnode(g, n))
-	UNMARK(&stk,n);
+	unmark(&stk, n);
 
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	if (MARKED(&stk,n))
+	if (marked(&stk, n))
 	    continue;
 	setPrefix(&name, pfx);
 	agxbprint(&name, "%" PRISIZE_T, c_cnt);
@@ -236,7 +243,7 @@ Agraph_t **ccomps(Agraph_t * g, int *ncc, char *pfx)
     freeStk (&stk);
     ccs = gv_recalloc(ccs, bnd, c_cnt, sizeof(Agraph_t*));
     agxbfree(&name);
-    *ncc = (int) c_cnt;
+    *ncc = c_cnt;
     return ccs;
 }
 
@@ -381,13 +388,12 @@ static void unionNodes(Agraph_t * dg, Agraph_t * g)
     }
 }
 
-static int clMarkFn (Agnode_t* n, int v)
-{
+static bool clMarkFn(Agnode_t *n, int v) {
     int ret;
-    if (v < 0) return clMark(n);
+    if (v < 0) return clMark(n) != 0;
     ret = clMark(n);
     clMark(n) = (char) v;
-    return ret;
+    return ret != 0;
 }
 
 typedef struct {
@@ -480,8 +486,7 @@ subGInduce(Agraph_t* g, Agraph_t * out)
  * is cloned within the subgraph. Each cloned cluster contains a record pointing
  * to the real cluster.
  */
-Agraph_t **cccomps(Agraph_t * g, int *ncc, char *pfx)
-{
+Agraph_t **cccomps(Agraph_t *g, size_t *ncc, char *pfx) {
     Agraph_t *dg;
     size_t n_cnt, c_cnt, e_cnt;
     agxbuf name = {0};
@@ -510,7 +515,7 @@ Agraph_t **cccomps(Agraph_t * g, int *ncc, char *pfx)
 
     c_cnt = 0;
     for (dn = agfstnode(dg); dn; dn = agnxtnode(dg, dn)) {
-	if (MARKED(&stk,dn))
+	if (marked(&stk, dn))
 	    continue;
 	setPrefix(&name, pfx);
 	agxbprint(&name, "%" PRISIZE_T, c_cnt);
@@ -551,7 +556,7 @@ Agraph_t **cccomps(Agraph_t * g, int *ncc, char *pfx)
     freeStk (&stk);
     ccs = gv_recalloc(ccs, ccs_length, c_cnt, sizeof(Agraph_t*));
     agxbfree(&name);
-    *ncc = (int) c_cnt;
+    *ncc = c_cnt;
     return ccs;
 }
 
@@ -572,7 +577,7 @@ int isConnected(Agraph_t * g)
 
     initStk(&stk, NULL, markFn);
     for (n = agfstnode(g); n; n = agnxtnode(g, n))
-	UNMARK(&stk,n);
+	unmark(&stk, n);
 
     n = agfstnode(g);
     cnt = dfs(g, agfstnode(g), NULL, &stk);
