@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -19,7 +19,6 @@
 #include <cgraph/exit.h>
 #include <cgraph/unused.h>
 #include <common/types.h>
-#include <common/memory.h>
 #include <common/globals.h>
 
 #include <sfdpgen/sparse_solve.h>
@@ -295,7 +294,6 @@ StressMajorizationSmoother SparseStressMajorizationSmoother_new(SparseMatrix A,
   /* solve a stress model to achieve the ideal distance among a sparse set of edges recorded in A.
      A must be a real matrix.
    */
-  StressMajorizationSmoother sm;
   int i, j, k, m = A->m, *ia, *ja, *iw, *jw, *id, *jd;
   int nz;
   double *d, *w, *lambda;
@@ -314,8 +312,7 @@ StressMajorizationSmoother SparseStressMajorizationSmoother_new(SparseMatrix A,
   ja = A->ja;
   a = A->a;
 
-
-  sm = MALLOC(sizeof(struct StressMajorizationSmoother_struct));
+  StressMajorizationSmoother sm = gv_alloc(sizeof(struct StressMajorizationSmoother_struct));
   sm->scaling = 1.;
   sm->data = NULL;
   sm->scheme = SM_SCHEME_NORMAL;
