@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -22,7 +22,7 @@
 */
 
 #define HIGHBITI	(~((~0u) >> 1))
-#define HIGHBITL	(~((~((Sfulong_t)0)) >> 1))
+#define HIGHBITL	(~((~0ull) >> 1))
 
 #define SFFMT_PREFIX	(SFFMT_MINUS|SFFMT_SIGN|SFFMT_BLANK)
 
@@ -40,7 +40,7 @@ static int imin(int a, int b) {
  */
 int sfprint(FILE *f, Sffmt_t *format) {
     int v = 0, n_s, base, fmt, flags;
-    Sflong_t lv;
+    long long lv;
     char *sp, *ssp, *endsp, *ep, *endep;
     int dot, width, precis, n, n_output = 0;
     int sign, decpt;
@@ -318,20 +318,22 @@ int sfprint(FILE *f, Sffmt_t *format) {
 	    size = -1;
 	    flags = (flags & ~SFFMT_TYPES) | SFFMT_TFLAG;
 	    goto loop_flags;
+	default: // continue with logic below
+	    break;
 	}
 
 	/* set the correct size */
 	if (flags & (SFFMT_TYPES & ~SFFMT_IFLAG)) {
 	    if (_Sftype[fmt] & (SFFMT_INT | SFFMT_UINT)) {
-		size = (flags & SFFMT_LLONG) ? sizeof(Sflong_t) :
+		size = (flags & SFFMT_LLONG) ? sizeof(long long) :
 		    (flags & SFFMT_LONG) ? sizeof(long) :
 		    (flags & SFFMT_SHORT) ? sizeof(short) :
 		    (flags & SFFMT_SSHORT) ? sizeof(char) :
-		    (flags & SFFMT_JFLAG) ? sizeof(Sflong_t) :
+		    (flags & SFFMT_JFLAG) ? sizeof(long long) :
 		    (flags & SFFMT_TFLAG) ? sizeof(ptrdiff_t) :
 		    (flags & SFFMT_ZFLAG) ? sizeof(size_t) : -1;
 	    } else if (_Sftype[fmt] & SFFMT_FLOAT) {
-		size = (flags & SFFMT_LDOUBLE) ? sizeof(Sfdouble_t) :
+		size = (flags & SFFMT_LDOUBLE) ? sizeof(long double) :
 		    (flags & (SFFMT_LONG | SFFMT_LLONG)) ?
 		    sizeof(double) : -1;
 	    }
@@ -359,7 +361,7 @@ int sfprint(FILE *f, Sffmt_t *format) {
 	    switch (_Sftype[fmt]) {
 	    case SFFMT_INT:
 	    case SFFMT_UINT:
-		if (FMTCMP(size, long, Sflong_t))
+		if (FMTCMP(size, long, long long))
 		     argv.l = va_arg(args, long);
 		else
 		    argv.i = va_arg(args, int);
@@ -465,7 +467,7 @@ int sfprint(FILE *f, Sffmt_t *format) {
 		(flags & ~(SFFMT_SIGN | SFFMT_BLANK | SFFMT_ZERO)) |
 		SFFMT_ALTER;
 	    if (sizeof(void*) > sizeof(int)) {
-		lv = (Sflong_t)(intptr_t)argv.vp;
+		lv = (long long)(intptr_t)argv.vp;
 		goto long_cvt;
 	    } else {
 		v = (int)(intptr_t)argv.vp;
@@ -508,42 +510,40 @@ int sfprint(FILE *f, Sffmt_t *format) {
 
 	  int_arg:
 	    if ((sizeof(long) > sizeof(int) || sizeof(void*) > sizeof(int))
-	        && FMTCMP(size, Sflong_t, Sflong_t)) {
+	        && FMTCMP(size, long long, long long)) {
 		lv = argv.ll;
 		goto long_cvt;
 	    } else if ((sizeof(long) > sizeof(int) || sizeof(void*) > sizeof(int))
-	               && FMTCMP(size, long, Sflong_t)) {
+	               && FMTCMP(size, long, long long)) {
 		if (fmt == 'd')
-		    lv = (Sflong_t) argv.l;
+		    lv = (long long)argv.l;
 		else
-		    lv = (Sflong_t) ((ulong) argv.l);
+		    lv = (long long)(ulong)argv.l;
 	      long_cvt:
 		if (lv == 0 && precis == 0)
 		    break;
 		if (lv < 0 && fmt == 'd') {
 		    flags |= SFFMT_MINUS;
 		    if (lv == HIGHBITL) {	/* avoid overflow */
-			lv = (Sflong_t) (HIGHBITL / base);
-			*--sp = _Sfdigits[HIGHBITL -
-					  ((Sfulong_t) lv) * base];
+			lv = (long long)(HIGHBITL / base);
+			*--sp = _Sfdigits[HIGHBITL - (unsigned long long)lv * base];
 		    } else
 			lv = -lv;
 		}
 		if (n_s < 0) {	/* base 10 */
-		    Sflong_t nv;
-		    sfucvt(lv, sp, nv, ssp, Sflong_t, Sfulong_t);
+		    long long nv;
+		    sfucvt(lv, sp, nv, ssp, long long, unsigned long long);
 		} else if (n_s > 0) {	/* base power-of-2 */
 		    do {
 			*--sp = ssp[lv & n_s];
-		    } while ((lv = ((Sfulong_t) lv) >> n));
+		    } while ((lv = (unsigned long long)lv >> n));
 		} else {	/* general base */
 		    do {
-			*--sp = ssp[((Sfulong_t) lv) % base];
-		    } while ((lv = ((Sfulong_t) lv) / base));
+			*--sp = ssp[(unsigned long long)lv % base];
+		    } while ((lv = (unsigned long long)lv / base));
 		}
 	    } else
-	    if (sizeof(short) < sizeof(int)
-		    && FMTCMP(size, short, Sflong_t)) {
+	    if (sizeof(short) < sizeof(int) && FMTCMP(size, short, long long)) {
 		if (ft && ft->extf && (ft->flags & SFFMT_VALUE)) {
 		    if (fmt == 'd')
 			v = (int) ((short) argv.h);
@@ -657,7 +657,7 @@ int sfprint(FILE *f, Sffmt_t *format) {
 	case 'E':
 	case 'f':
 	    if (!(ft && ft->extf && (ft->flags & SFFMT_VALUE)) ||
-		    FMTCMP(size, double, Sfdouble_t))
+		    FMTCMP(size, double, long double))
 		 dval = argv.d;
 	    else
 		dval = (double) argv.f;
