@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -324,7 +324,7 @@ static void write_xdots (char * val, GVJ_t * job, state_t* sp)
 
     cmds = parseXDot(val);
     if (!cmds) {
-	agerr(AGWARN, "Could not parse xdot \"%s\"\n", val);
+	agwarningf("Could not parse xdot \"%s\"\n", val);
 	return;
     }
 
@@ -437,8 +437,19 @@ static bool write_subgs(Agraph_t *g, GVJ_t *job, bool top, state_t *sp) {
     return true;
 }
 
-static int agseqasc(Agedge_t **lhs, Agedge_t **rhs)
-{
+static int agseqasc(const void *x, const void *y) {
+// Suppress Clang/GCC -Wcast-qual warning. Casting away const here is acceptable
+// as the later usage is const. We need the cast because the macros use
+// non-const pointers for genericity.
+#ifdef __GNUC__
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-qual"
+#endif
+  Agedge_t **lhs = (Agedge_t **)x;
+  Agedge_t **rhs = (Agedge_t **)y;
+#ifdef __GNUC__
+#pragma GCC diagnostic pop
+#endif
     Agedge_t *e1 = *lhs;
     Agedge_t *e2 = *rhs;
 
@@ -497,7 +508,7 @@ static int write_edges(Agraph_t *g, GVJ_t *job, bool top, state_t *sp) {
         }
     }
 
-    qsort(edges, count, sizeof(Agedge_t *), (qsort_cmpf)agseqasc);
+    qsort(edges, count, sizeof(Agedge_t *), agseqasc);
 
     gvputs(job, ",\n");
     indent(job, sp->Level++);
@@ -623,7 +634,7 @@ static void insert (Dt_t* map, char* name, int v)
 
     if (ip) {
 	if (ip->v != v)
-	    agerr(AGWARN, "Duplicate cluster name \"%s\"\n", name);
+	    agwarningf("Duplicate cluster name \"%s\"\n", name);
 	return;
     }
     ip = gv_alloc(sizeof(intm));
