@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -399,8 +399,6 @@ tclGdCreateCmd(Tcl_Interp * interp, int argc, Tcl_Obj * CONST objv[])
 {
     int w, h;
     gdImagePtr im = NULL;
-    FILE *filePtr;
-    ClientData clientdata;
     char *cmd;
     Tcl_Obj *result;
     int fileByName;
@@ -440,9 +438,14 @@ tclGdCreateCmd(Tcl_Interp * interp, int argc, Tcl_Obj * CONST objv[])
     } else {
 	char *arg2 = Tcl_GetString(objv[2]);
 	fileByName = 0;		/* first try to get file from open channel */
+	FILE *filePtr = NULL;
+#if !defined(_WIN32)
+	ClientData clientdata;
 	if (Tcl_GetOpenFile(interp, arg2, 0, 1, &clientdata) == TCL_OK) {
 	    filePtr = (FILE *) clientdata;
-	} else {
+	}
+#endif
+	if (filePtr == NULL) {
 	    /* Not a channel, or Tcl_GetOpenFile() not supported.
 	     *   See if we can open directly.
 	     */
@@ -521,8 +524,6 @@ static int
 tclGdWriteCmd(Tcl_Interp * interp, int argc, Tcl_Obj * CONST objv[])
 {
     gdImagePtr im;
-    FILE *filePtr;
-    ClientData clientdata;
     const char *cmd, *fname;
     int fileByName;
     int arg4;
@@ -553,10 +554,15 @@ tclGdWriteCmd(Tcl_Interp * interp, int argc, Tcl_Obj * CONST objv[])
 
     /* Get the file reference. */
     fileByName = 0;		/* first try to get file from open channel */
+    FILE *filePtr = NULL;
+#if !defined(_WIN32)
+    ClientData clientdata;
     if (Tcl_GetOpenFile(interp, fname, 1, 1, &clientdata)
 	== TCL_OK) {
 	filePtr = (FILE *) clientdata;
-    } else {
+    }
+#endif
+    if (filePtr == NULL) {
 	/* Not a channel, or Tcl_GetOpenFile() not supported.
 	 *   See if we can open directly.
 	 */
